/* 
 ***********************************************************
 $SNARK_Header$
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/trm6.h $
 $LastChangedRevision: 122 $
 $Date: 2014-07-09 18:18:59 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Abstract class for distance measurements

 */

#ifndef Distance_Measure
#define Distance_Measure

#include "blkdta.h"

class DistanceMeasure {

// attributes
private:
	char* distanceName;

// methods
public:
	DistanceMeasure(char* distanceNameIn);
	virtual ~DistanceMeasure();

	/**
	 * Calculates the distance for the provided reconstruction.
	 *
	 * input:	recon		the reconstruction, from which the distance should be calculated
	 * 			list		a allocated list with lenght = max pixel intersected by one ray
	 * 			weight		a allocated list with lenght = max pixel intersected by one ray
	 */
	REAL calculateDistance(REAL* recon, INTEGER* list, REAL* weight);

	/**
	 * Calculates the distance for the provided reconstruction.
	 *
	 * input:	reconPixel	the reconstruction, from which the distance should be calculated
	 * 			list		a allocated list with lenght = max pixel intersected by one ray
	 * 			weight		a allocated list with lenght = max pixel intersected by one ray
	 */
	virtual REAL calculateDistanceForPixel(REAL* reconPixel, INTEGER* list,
	REAL* weight) = 0;

	/**
	 * Calculates the distance for the provided reconstruction.
	 *
	 * input:	reconBlob	the reconstruction, from which the distance should be calculated
	 * 			list		a allocated list with lenght = max pixel intersected by one ray
	 * 			weight		a allocated list with lenght = max pixel intersected by one ray
	 */
	virtual REAL calculateDistanceForBlob(REAL* reconBlob, INTEGER* list,
	REAL* weight) = 0;

	/**
	 * Returns the name of the distance measure
	 */
	char* getName();
};

#endif
