/*
 ***********************************************************
 $SNARK_Header$
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/trm6.cpp $
 $LastChangedRevision: 122 $
 $Date: 2014-07-09 18:18:59 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <math.h>
#include "DistanceMeasureWSQD.h"
#include "anglst.h"
#include "pseudo.h"
#include "consts.h"

using namespace std;

DistanceMeasureWSQD::DistanceMeasureWSQD() :
		DistanceMeasurePixel("weighted squared distance") {
}
DistanceMeasureWSQD::~DistanceMeasureWSQD() {
}

REAL DistanceMeasureWSQD::calculateDistanceForPixel(REAL* reconPixel,
INTEGER* list, REAL* weight) {

	// variables needed during computation of the current residual

	REAL theta = 0;
	REAL sinth = 0;
	REAL costh = 0;

	BOOLEAN linef = true;

	REAL raysum = 0;
	REAL psum = 0;
	INTEGER numb = 0;
	REAL snorm = 0;
	REAL raylen = 0;
	REAL wsqd = 0;

	// compute current residual
	for (int np = 0; np < GeoPar.prjnum; np++) {
		REAL rinc = GeoPar.pinc;
		Anglst.getang(np, &theta, &sinth, &costh);

		if (GeoPar.vri) {
			rinc = GeoPar.pinc * MAX0(fabs(sinth), fabs(costh));
		}

		for (int nr = GeoPar.fusray; nr <= GeoPar.lusray; nr++) {
			raysum = Anglst.prdta(np, nr);
			psum = pseudo(reconPixel, np, nr, list, weight, &numb, &snorm,
					linef, TRUE);

			if (numb != 0) {
				// pseudo already evaluates the values for weight/number of affected pixel
				// --> can be reused
				// ATTENTION: no other operations on weight/numb are allowed between pseudo and this calculation
				raylen = 0.0;
				for (int i = 0; i < numb; i++) {
					raylen += weight[i];
				}

				if (raylen > Consts.zero) {
					if (linef && !GeoPar.line)
						raysum /= rinc;
					if (!linef && GeoPar.line)
						raysum *= rinc;
					wsqd += pow(psum - raysum, 2) / raylen;
				}
			}
		}
	}

	return wsqd;
}
