/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/art.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS IS A GENERAL INPLEMENTATION OF ALGEBRAIC RECONSTRUUCTION
 THIS SUBROUTINE IS CAPABLE OF EXECUTION THE FOLLOWING METHODS:

 ART3 FROM
 A RELAXATION METHOD FOR RECONSTRUCTING OBJECTS FROM NOISY
 X-RAY, G.T. HERMAN, MATH. PROG.,V8(1975),1-19.
 ART4 FROM
 ITERATIVE QUADRATIC OPTIMIZATION WITH APPLICATION IN DI-
 AGNOSTIC RADIOLOGY, G.T. HERMAN AND A. LENT, SUNYAB
 DEPARTMENT OF COMPUTER SCIENCE, INTERNAL RE-
 PORT NO. 122.
 BAYESIAN FROM
 A COMPUTER IMPLEMENTATION OF A BAYESIAN ANALYSIS OF IMAGE
 RECONSTRUCTION, G.T. HERMAN , A. LENT, H. HURWITZ, AND
 H.P. LUNG, SUNYAB DEPARTMENT OF COMPUTER SCIENCE INTERNAL
 REPORT NO. 134.
 WITH THE FOLLOWING OPTIONAL MODIFICATIONS ON THE RECONSTRUCTION
 ART2-WAY CONSTRAINT
 REFERENCE IS THE SAME AS ART4.
 BINARY PATTERNS (BART)
 RECONSTRUCTION OF BINARY PATTERNS FROM A FEW PROJECTIONS,
 G.T. HERMAN, INTERNATIONAL COMPUTING SYMPOSIUM 1973,
 371-379.

 IMPORTANT VARIABLES:
 KOUNT - NUMBER OF RAYS USED IN EACH ITERATION
 HALFWY- EQUALS TO (UPPER+LOWER)/2.0
 THTOL - EQUALS TO (UPPER-LOWER)/1000.0
 THRLO - (1) STARTING FROM LOWER, AND IS INCREMENTED BY
 (UPPER-LOWER)/(KOUNT*2**(ITER+1)) AFTER EACH RECON-
 STRUCTION LOOP IN THIS ROUTINE WHEN BART IS
 SPECIFIED (BART WILL BE EXPLAINED LATER)
 (2) EQUALS TO LOWER OTHERWISE
 THRUP - (1) STARTING FROM UPPER, AND IS DECREMENTED BY THE SAME
 AMOUNT AS INDICATED IN THRLO (1) AFTER EACH RECON-
 STRUCTION LOOP WHEN BART IS SPECIFIED
 (2) EQUALS TO UPPER OTHERWISE

 THE FOLLOWING KEYWORDS ARE AVAILABLE FOR THE USER TO IMPLEMENT
 THIS SUBROUTINE :

 ART3 -
 RECONSTRUCTION WILL BE BASED ON ART3 METHOD
 ART4 -
 RECONSTRUCTION WILL BE BASED ON ART4 METHOD
 BAYESIAN -
 RECONSTRUCTION WILL BE BASED ON BAYESIAN METHOD
 CONSTRAINT -
 EITHER "ART2" OR "BOUND" OR "BART" MUST BE SPECIFIED AFTER
 THIS KEYWORD.  WHEN "ART2" IS SPECIFIED, THE ART2 WAY
 CONSTRAINT IS USED IN THE RECONSTRUCTION.  IF "BOUND" IS
 SPECIFIED, THE RECONSTRUCTION IS BOUNDED BY THE BOUNDARIES
 "UPPER" AND "LOWER".  WHEN "BART" IS SPECIFIED, THE RECON-
 STRUCTED PICTURE IS ASSUMED TO BE A BINARY PATTERN(WITH
 GRAY LEVELS "UPPER" AND "LOWER").  AND THE PICTURE ELEMENTS AR
 COMPARED WITH THRLO, THRUP, AND HALFWY WITH ASSOCIATED CON-
 TRAINT.  WHEN LOFL IS ON, IF THE ORIGINAL PICTURE ELEMENT IS
 LESS THAN THRLO AND ITS RECONSTRUCTION IS GREATER THAN HALFWY,
 THEN THIS ELEMENT IS CHANGED TO HALFWY-THTOL.  WHEN UPFL IS
 ON, IF THE ORIGINAL PICTURE ELEMENT IS GREATER THAN THRUP
 AND ITS RECONSTRUCTION IS LESS THAN HALFWY THEN IT IS
 CHANGED TO HALFWY+THTOL.
 RELAX -
 THE USER CAN SPECIFY THE VALUE OF THE RELAXATION PARAMETER TO
 BE USED IN THE RECONSTRUCTION BY THIS OPTION.  THE RELAX-
 ATION PARAMETER CAN BE EITHER CONSTANT OR VARIABLE.  IF A
 CONSTANT IS DESIRED THE USER MUST SPECIFY "RELAX CONSTANT"
 FOLLOWED BY A FLOATING POINT NUMBER.  IF THE PARAMETER IS
 VARIABLE THEN "RELAX VARIABLE" MUST BE SPECIFIED, AND A
 FUNCTION RSET(RECON,ITER,NP,NR,I,RAYSUM,LIST,WEIGHT)
 FOR CALCULATING THE VARIABLE PARAMETER MUST
 BE SUPPLIED BY THE USER, WHERE THE CALCULATED
 RELAXATION PARAMETER IS RETURNED BY ASSIGNING THE VALUE TO
 RSET.  THE DEFAULT IS CONSTANT 1.0.
 CONRELAX -
 THE MODIFIERS FOR CONRELAX IS THE SAME AS OPTION "RELAX".
 IN THE VARIABLE CASE THE NAME OF THE USER SUPPLIED
 FUNCTION IS CRSET(RECON,ITER,NP,NR,I,RAYSUM,LIST,WEIGHT).
 NORM -
 AN INTEGER MUST BE GIVEN AFTER "NORM".  THE USER HAS HIS
 FREEDOM OF CHOOSING THE TYPE OF NORM (>0) USED IN THE ITER-
 ATIVE STEP BY SPECIFYING THE INTEGER NUMBER OF THE NORM.
 THE DEFAULT IS 2-NORM.
 STEPS -
 AN INTEGER MUST BE SPECIFIED AFTER THIS KEYWORD, AND THE
 VALUE OF THE INTEGER IS THE NUMBER OF RAYS USED FOR THE RE-
 CONSTRUCTION FOR EACH SNARK05 ITERATION.  THE RAYS CHOSEN
 ONE AFTER ANOTHER ARE DETERMINED BY THE SNARK05 "SELECT"
 COMMAND.  THE DEFAULT IS A NUMBER EQUIVALENT TO
 PRJNUM*NRAYS.
 NOMLZ -
 THE FINAL RECONSTRUCTION OF EACH ITERATION
 WILL BE NORMALIZED SO THAT THE AVERAGE DENSITY OF THE PICT-
 URE EQUALS TO AVEDEN; OTHERWISE THE RECONSTRUCTION IS
 NOT NORMALIZED.
 TOLERANCE -
 THERE ARE THREE POSSIBILITIES TO DEFINE THE TOLERANCE FOR
 ART3 AND ART4:
 (1) "TOLERANCE FIXED T", WHERE T IS A USER SUPPLIED FL-
 OATING POINT NUMBER, AND IT IS USED AS THE TOLERANCE DUR-
 ING THE RECONSTRUCTION.
 (2) "TOLERANCE VARIABLE", THE TOLERANCE IS CHANGED FOR EACH
 SNARK05 ITERATIVE STEP.  THE USER HAS TO SUPPLY A FUNC-
 TION TSET(RECON,ITER,NP,NR,I,RAYSUM,LIST,WEIGHT)
 SUCH THAT THE CALCULATED TOLERANCE IS ASSIGNED TO
 TSET AT EXIT.
 (3) "TOLERANCE NOISE T", THE TOLERANCE IS DEPENDENT ON THE
 NOISE IN THE PROJECTION DATA.  IT IS CALCULATED BY
 T*SQRT(VARIAN) WHERE VARIAN IS OBTAINED BY
 CALLING FUNCTION ERRFAC.

 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "fourie.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "errfac.h"
#include "errpar.h"
#include "pick.h"
#include "pseudo.h"

#include "art.h"

INTEGER art_class::Init()
{
	//Xalg1User3.Open(); //Ran, bug 266
	//ArtIn.delta_1 = NULL; //wei, 3/2005 //Ran, bug 266
	return 0;
}

INTEGER art_class::Reset()
{
	// Xalg1User3.Close(); //Ran, bug 266
	// if (ArtIn.delta_1 != NULL) delete[] ArtIn.delta_1; //wei,3/2005 //Ran, bug 266
	if (!ArtIn.art3f)
		delete[] ArtIn.totalRays; //wei, bug 266
	return 0;
}

BOOLEAN art_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{

	static INTEGER iflagr;
	static INTEGER iflagt;
	static INTEGER iflagc;
	static REAL thrinc;
	static REAL dist;
	static REAL minprd;
	static REAL rinc2;
	INTEGER i;
	INTEGER np;
	INTEGER nr;
	REAL raysum;
	REAL theta;
	REAL sinth;
	REAL costh;
	REAL v;
	REAL psrsum;
	INTEGER numb;
	REAL snorm;
	INTEGER j;
	REAL diff;
	REAL abdiff;

	REAL sum;
	REAL temp;
	REAL averec;
	REAL avedif;

	BOOLEAN erf = true;
	BOOLEAN erflag;
	static BOOLEAN flag; //may not need to be static hstau

	REAL pictk;

	INTEGER area;

	REAL pof2;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	if (iter == 1)
	{
		flag = FALSE;
		iflagr = 1;
		iflagt = 1;
		iflagc = 1;

		readin(recon, list, weight, &erflag);

		if (erflag)
		{
			return TRUE;
		}

		ArtIn.thrlo = Modefl.lower;
		ArtIn.thrup = Modefl.upper;
		thrinc = 0.0;
		ArtIn.halfwy = (Modefl.upper + Modefl.lower) / (REAL) 2.0;
		ArtIn.thtol = (Modefl.upper - Modefl.lower) / (REAL) 1000.0;
		dist = Modefl.upper - Modefl.lower;

		if (ArtIn.noisef)
		{
			minprd = 0.0;
			Fourie.rinc = GeoPar.pinc;
			rinc2 = GeoPar.pinc * GeoPar.pinc;
			errpar(erf);
		}
	} // iter == 1

	if (ArtIn.bartf)
	{

		pof2 = 1;
		for (i = 0; i < iter + 1; i++)
			pof2 *= 2.0;

		thrinc = dist / (ArtIn.kount * pof2);
	}

	// ONE ITERATION

	for (i = 0; i < ArtIn.kount; i++)
	{
		pick(&np, &nr);
		raysum = Anglst.prdta(np, nr);

		if (!ArtIn.rconsf)
		{

			// GET VARIABLE RELAXATION PARAMETER

			ArtIn.relax = rset(recon, iter, np, nr, iflagr, raysum, list,
					weight, &flag);
			if (flag)
			{
				return TRUE;
			}
		}
		if (ArtIn.noisef)
		{

			// GET VARIABLE TOLERANCE BASED ON NOISE IN PROJECTION

			if (!(GeoPar.line || GeoPar.uni))
			{
				Anglst.getang(np, &theta, &sinth, &costh);
				Fourie.rinc = GeoPar.pinc * MAX0(sinth, costh);
				rinc2 = Fourie.rinc * Fourie.rinc;
			}
			v = errfac(raysum, Fourie.rinc, rinc2);
			ArtIn.toler = ArtIn.ct * (REAL) sqrt(v);
		}
		else
		{
			if (!ArtIn.tolerf)
			{

				// GET VARIABLE TOLERANCE

				ArtIn.toler = tset(recon, iter, np, nr, iflagt, raysum, list,
						weight, &flag);
				if (flag)
				{
					return TRUE;
				}
				if (ArtIn.toler < 0.0)
				{
					fprintf(output,
							"\n          ***tolerance calculated at iter= %5i  np= %5i  nr= %5i  tolerance= %9.5f***",
							iter, np, nr, ArtIn.toler); //hstau

					fprintf(output,
							"\n          ***error - tolerance can not be less than 0.0***"); //hstau
					return TRUE;
				}
			}
		}

		if (!ArtIn.crf)
		{

			// GET VARIABLE CR

			ArtIn.cr = crset(recon, iter, np, nr, iflagc, raysum, list, weight,
					&flag);
			if (flag)
			{
				return TRUE;
			}
		}

		if (Blob.pix_basis)
		{
			psrsum = pseudo(recon, np, nr, list, weight, &numb, &snorm,
					GeoPar.line, ArtIn.art2f);
		}
		else
		{
			psrsum = Blob.bpseudo(recon, np, nr, list, weight, &numb, &snorm,
					GeoPar.line, ArtIn.art2f);
		}

		if (numb != 0)
		{

			// CALCULATE P-NORM WHEN P IS OTHER THAN 2

			if (ArtIn.p != 2)
			{
				snorm = 0.0;
				for (j = 0; j < numb; j++)
				{
					snorm += (REAL) pow(weight[j], ArtIn.p);
				}
			}
			if (snorm > Consts.zero)
			{
				diff = raysum - psrsum;
				abdiff = (REAL) fabs(diff);

				// BACKPROJECTION
				bkproj(recon, diff, abdiff, ArtIn.totalRays, np, nr, list,
						weight, numb, snorm);

			}
		}
		// bug 222 - swr - 04/08/07
		ArtIn.thrlo += thrinc;
		ArtIn.thrup -= thrinc;
	}

	if (ArtIn.nomlf)
	{

		// NORMALIZATION

		sum = 0.0;
		for (i = 0; i < area; i++)
		{
			temp = recon[i];
			temp = clip(temp, ArtIn.thrlo, ArtIn.thrup, 1.0);
			if (!Blob.pix_basis)
			{ //if blobs
				if (i % 2 == Blob.pr)
				{
					sum += temp;
				}
			}
			else
			{
				sum += temp;
			}
		}

		averec = sum / float(area);
		if (!Blob.pix_basis)
		{
			averec *= 2.0;
		}
		avedif = GeoPar.aveden - averec; //aveden in both basis coincide

		// IN THE FOLLOWING LOOP OF 820 THE COMMENTED LINE WAS
		// REPLACED WITH THE CODE INSIDE LOOP 820....10/13/87
		// Dr. GABOR T HERMAN

		for (i = 0; i < area; i++)
		{

			pictk = recon[i] + avedif;
			if (!ArtIn.art2f)
			{
				pictk = clip(pictk, ArtIn.thrlo, ArtIn.thrup, 1.0);
				if (ArtIn.bartf)
				{
					if (Modefl.lofl && (recon[i] <= ArtIn.thrlo)
							&& (pictk >= ArtIn.halfwy))
						pictk = ArtIn.halfwy - ArtIn.thtol;

					if (Modefl.upfl && (recon[i] >= ArtIn.thrup)
							&& (pictk <= ArtIn.halfwy))
						pictk = ArtIn.halfwy + ArtIn.thtol;
				}
			}
			recon[i] = pictk;
		}
	}

	return FALSE;
}

