/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/art_readin.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 READ IN INPUT OPTIONS AND SET DEFAULTS
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"

#include "int2str.h"
#include "infile.h"
#include "raysel.h"

#include "art.h"

/// !!! list & weight never used !!!

void art_class::readin(REAL* recon, INTEGER* list, REAL* weight, BOOLEAN* flag)
{
	static const INTEGER bound = CHAR2INT('b', 'o', 'u', 'n');
	static const INTEGER vari = CHAR2INT('v', 'a', 'r', 'i');
	static const INTEGER noise = CHAR2INT('n', 'o', 'i', 's');
	static const INTEGER art2 = CHAR2INT('a', 'r', 't', '2');
	static const INTEGER cons = CHAR2INT('c', 'o', 'n', 's');
	static const INTEGER bart = CHAR2INT('b', 'a', 'r', 't');
	static const INTEGER fixed = CHAR2INT('f', 'i', 'x', 'e');

	static const INTEGER norm = CHAR2INT('n', 'o', 'r', 'm');
	static const INTEGER rela = CHAR2INT('r', 'e', 'l', 'a');
	static const INTEGER art4 = CHAR2INT('a', 'r', 't', '4');
	static const INTEGER tole = CHAR2INT('t', 'o', 'l', 'e');
	static const INTEGER conr = CHAR2INT('c', 'o', 'n', 'r');
	static const INTEGER step = CHAR2INT('s', 't', 'e', 'p');
	static const INTEGER noml = CHAR2INT('n', 'o', 'm', 'l');
	static const INTEGER baye = CHAR2INT('b', 'a', 'y', 'e');
	static const INTEGER art3 = CHAR2INT('a', 'r', 't', '3');

	int i;

	BOOLEAN eol;
	BOOLEAN boundf;

	INTEGER word;
	//REAL* delta; ///!!!!

	// DEFAULT INITIALIZATION

	ArtIn.p = 2;
	ArtIn.relax = 1.0;
	ArtIn.toler = 0.0;
	//ArtIn.kount = GeoPar.prjnum * GeoPar.usrays; // change nrays to usrays. hstau 7/2003
	{
		if (RaySel.useray)  // select user
			ArtIn.kount = GeoPar.prjnum * GeoPar.usrays;
		else
			//select snark
			ArtIn.kount = GeoPar.prjnum * GeoPar.snrays;
	}  // set value to kount for select user or select snark. Wei   11/2004

	ArtIn.npin = -1;
	ArtIn.rconsf = TRUE;
	ArtIn.cr = 1.0;
	ArtIn.crf = TRUE;
	ArtIn.art2f = FALSE;
	ArtIn.art3f = FALSE;
	ArtIn.art4f = FALSE;
	ArtIn.tolerf = TRUE;
	ArtIn.nomlf = FALSE;
	boundf = FALSE;
	ArtIn.bartf = FALSE;
	ArtIn.bayef = FALSE;
	ArtIn.noisef = FALSE;
	*flag = FALSE;

	INTEGER exp0[3] =
	{ art3, art4, baye };

	word = InFile.getwrd(TRUE, &eol, exp0, 3);

	if (eol)
	{
		fprintf(output,
				"\n          **** error - one and only one of the following options must be specified : art3, art4, bayesian***");
		*flag = TRUE;
		return;
	}

	if (word == art3)
	{
		ArtIn.art3f = TRUE;
		fprintf(output, "\n          art3 method");
	}

	else if (word == art4)
	{
		ArtIn.art4f = TRUE;
		fprintf(output, "\n          art4 method");
	}

	else
	{ //word == baye
		ArtIn.bayef = TRUE;
		ArtIn.snr = InFile.getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output,
					"\n          **** error - must specify the snr in Bayesian option***");
			*flag = TRUE;
			return;
		}

		ArtIn.snr2 = ArtIn.snr * ArtIn.snr;
		if (ArtIn.snr2 <= Consts.zero)
		{
			fprintf(output,
					"\n          **** error - snr**2= %10.6f must be greater than %12.4e***",
					ArtIn.snr2, Consts.zero);

			*flag = TRUE;
			return;
		}

		fprintf(output, "\n          Bayesian method with  snr = %9.5f",
				ArtIn.snr);
	}  // baye

	INTEGER exp1[3] =
	{ rela, norm, tole };
	word = InFile.getwrd(FALSE, &eol, exp1, 3);

	if (eol)
	{
		fprintf(output, "\n          relaxation parameter is 1.0");
		fprintf(output, "\n          norm is 2");
		fprintf(output, "\n          tolerance is 0.0");
		goto L20;
		// read next line
	}

	if (word == rela)
	{
		INTEGER exp2[2] =
		{ cons, vari };
		word = InFile.getwrd(FALSE, &eol, exp2, 2);

		if (eol)
		{
			fprintf(output,
					"\n          **** error - with the option relaxation, one and only one of the following options must be specified : constant or variable***");
			*flag = TRUE;
			return;
		}

		if (word == cons)
		{
			ArtIn.relax = InFile.getnum(FALSE, &eol);

			if (eol)
			{
				fprintf(output,
						"\n          **** error - must specify the relaxation constant***");
				*flag = TRUE;
				return;
			}
			else
			{
				fprintf(output, "\n          relaxation parameter is %9.4f",
						ArtIn.relax);
			}
		}

		else
		{  //relax=vari
			ArtIn.rconsf = FALSE;
			fprintf(output, "\n          relaxation parameter is variable");
		}

		INTEGER exp1a[2] =
		{ norm, tole };
		word = InFile.getwrd(FALSE, &eol, exp1a, 2);

		if (word == norm)
			goto L10;
		else if (word == tole)
			goto L15;
		else
		{
			fprintf(output, "\n          norm is 2");
			fprintf(output, "\n          tolerance is 0.0");
			goto L20;
			// eol: read next line
		}
	} // word == rela

	else if (word == norm)
	{
		L10: ArtIn.p = InFile.getint(FALSE, &eol);

		if (eol)
		{
			fprintf(output,
					"\n          **** error - must specify the norm***");
			*flag = TRUE;
			return;
		}
		else if (ArtIn.p < 0)
		{
			fprintf(output, "\n          norm cannot be negative");
			*flag = TRUE;
			return;
		}
		else
		{
			fprintf(output, "\n          norm is %5i", ArtIn.p);
		}

		INTEGER exp1b[1] =
		{ tole };
		word = InFile.getwrd(FALSE, &eol, exp1b, 1);

		if (word == tole)
			goto L15;
		else
		{
			fprintf(output, "\n          tolerance is 0.0");
			goto L20;
			// eol: read next line
		}
	}  // word == norm

	else
	{ //word = tole

		L15:
		INTEGER exp3[3] =
		{ fixed, vari, noise };
		word = InFile.getwrd(FALSE, &eol, exp3, 3);

		if (eol)
		{
			fprintf(output,
					"\n          **** error - with the option tolerance, one and only one of the following options must be specified : fixed, variable, or noise***");
			*flag = TRUE;
			return;
		}

		if (word == fixed)
		{
			ArtIn.toler = InFile.getnum(FALSE, &eol);

			if (eol)
			{
				fprintf(output,
						"\n          **** error - must specify tolerance ***");
				*flag = TRUE;
				return;
			}

			else if (ArtIn.toler < 0.0)
			{
				fprintf(output,
						"\n          **** error - tolerance can not be less than 0.0***");
				*flag = TRUE;
				return;
			}

			else
			{
				fprintf(output, "\n          tolerance is %9.4f", ArtIn.toler);
			}
		} // fixed

		else if (word == vari)
		{
			ArtIn.tolerf = FALSE;
			fprintf(output, "\n          tolerance is variable");
		}

		else
		{ // noise
			ArtIn.noisef = TRUE;
			ArtIn.ct = InFile.getnum(FALSE, &eol);

			if (eol)
			{
				fprintf(output,
						"\n          **** error - must specify tolerance ***");
				*flag = TRUE;
				return;
			}

			else if (ArtIn.ct < 0.0)
			{
				fprintf(output,
						"\n          **** error - tolerance can not be less than 0.0***");
				*flag = TRUE;
				return;
			}

			else
			{
				fprintf(output,
						"\n          tolerance is data dependent, noise factor%9.4f",
						ArtIn.ct);
			}
		} //noise
	} //word == tole

	L20:

	INTEGER exp4[1] =
	{ cons };

	word = InFile.getwrd(TRUE, &eol, exp4, 1);

	if (eol)
	{
		fprintf(output,
				"\n          **** error - keyword constraint is missing***");
		*flag = TRUE;
		return;
	}

	INTEGER exp5[3] =
	{ art2, bound, bart };

	word = InFile.getwrd(FALSE, &eol, exp5, 3);

	if (eol)
	{
		fprintf(output,
				"\n          **** error - with the option constraint, one and only one of the following options must be specified : art2, bound, or bart***");
		*flag = TRUE;
		return;
	}

	if (word == art2)
	{
		ArtIn.art2f = TRUE;
		fprintf(output, "\n          constraint art2 way");
	}

	else if (word == bound)
	{
		boundf = TRUE;
		fprintf(output, "\n          constraint set to boundaries");
	}

	else
	{ //word == bart
		ArtIn.bartf = TRUE;
		if (!(Modefl.lofl && Modefl.upfl))
		{
			fprintf(output,
					"\n          *** error - both upfl and lofl must be on when bart is specified***");
			*flag = TRUE;
			return;
		}

		fprintf(output, "\n          constraint bart");

	}  //bart

	INTEGER exp6[3] =
	{ conr, step, noml };
	word = InFile.getwrd(FALSE, &eol, exp6, 3);

	if (eol)
	{
		fprintf(output, "\n          relaxation constraints with cr = 1.0");
		fprintf(output, "\n          picture is not normalized");
		goto L40;
		// finished reading input
	}
	if (word == conr)
	{
		INTEGER exp7[2] =
		{ cons, vari };
		word = InFile.getwrd(FALSE, &eol, exp7, 2);

		if (eol)
		{
			fprintf(output,
					"\n          **** error - with the option conrelax, one and only one of the following options must be specified : constant or variable***");
			*flag = TRUE;
			return;
		}

		if (word == cons)
		{
			ArtIn.cr = InFile.getnum(FALSE, &eol);

			if (eol)
			{
				fprintf(output,
						"\n          **** error - must specify constraint relaxation constant***");
				*flag = TRUE;
				return;
			}
			fprintf(output, "\n          relax constraints with cr= %9.4f",
					ArtIn.cr);
		}
		else
		{ //variable relax constaints
			ArtIn.crf = FALSE;
			fprintf(output, "\n          relax constraints with variable cr");
		}

		INTEGER exp6a[2] =
		{ step, noml };
		word = InFile.getwrd(FALSE, &eol, exp6a, 2);

		if (word == step)
			goto L30;
		else if (word == noml)
			goto L35;
		else
		{
			fprintf(output, "\n          picture is not normalized");
			goto L40;
			// eol: finished
		}
	} // constraint relaxation

	else if (word == step)
	{
		L30: ArtIn.kount = InFile.getint(FALSE, &eol);

		if (eol)
		{
			fprintf(output, "\n          **** error - must specify kount***");
			*flag = TRUE;
			return;
		}
		else if (ArtIn.kount <= 0)
		{
			fprintf(output,
					"\n          **** error - kount can not be less than 0 **");
			*flag = TRUE;
			return;
		}

		INTEGER exp6b[1] =
		{ noml };
		word = InFile.getwrd(FALSE, &eol, exp6b, 1);

		if (word == noml)
			goto L35;
		else
		{
			fprintf(output, "\n          picture is not normalized");
			goto L40;
			// eol: finished
		}
	}  // word = step

	L35: //nomlz
	ArtIn.nomlf = TRUE;
	fprintf(output,
			"\n          picture is normalized to %9.4f after each iteration",
			GeoPar.aveden);

	L40: if (!ArtIn.art3f)
	{

		// ALLOCATE AND INITIALIZE DUAL VARIABLES IN ART4
		// OR IN BAYESIAN

		//Ran, bug 266
		ArtIn.totalRays = new REAL[GeoPar.nrays * GeoPar.prjnum];
		for (i = 0; i < (GeoPar.nrays * GeoPar.prjnum); i++)
		{
			ArtIn.totalRays[i] = 0.0;
		}
	}

	fprintf(output, "\n          %5i rays are used for each iteration",
			ArtIn.kount);

	if (Modefl.lofl)
	{
		fprintf(output, "\n          lower constraint = %9.4f", Modefl.lower);
	}
	if (Modefl.upfl)
	{
		fprintf(output, "\n          upper constraint = %9.4f", Modefl.upper);
	}

	return;
}

