/*      
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bh_correction.h $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 * Description:
 *
 * The class bh_correction is responsible for performing beam hardening
 * correction for polychromatic input data. Given the prjfil that was calculated
 * based on weighted average of attenuations at different energy levels and 
 * corrections coefficients [1,2] it approximates monochromatic projection data 
 * which is then used by the reconstruction algorithms.
 *
 * 
 *
 * References:
 * [1] G. T. Herman, Image Reconstruction from Projections, 1980
 * [2] SNARK08 manual
 *
 */

#include "creacm.h"
#include "spctrm.h"
#include "uiod.h"

#include "snark.h"
#include "recfile.h"
#include "file11.h"
#include "projfile.h"
#include "infile.h"

#ifndef _BH_CORRECTION_H
#define	_BH_CORRECTION_H

//jk 8/26/08   for use with beam hardening correction by non-parametric 
// function
class bin
{
public:
	unsigned int noOfItems;
	double aveOfValues;
	double midValue;

};

class binCollection
{
public:

};

class point
{
public:
	REAL x;REAL y;
};

class bh_correction
{
private:

public:

	BOOLEAN bhc;BOOLEAN bhc_poly;

	char tempDir[256];

	INTEGER nerg;       //number of energy levels read from the input file
	//should be the same as Spectrm.nergy

	INTEGER noIter;          //number of iterations

	//POLY
	REAL * a;           //array of polynomial coefficients
	INTEGER noDegree;       //degree of the polynomial
	INTEGER noDegreeConv;   //degree of the polynomials for converting
	//phantom data from energy[0] to other energy levels
	REAL * b[7];        //array of polynomial coefficients for converting
						//phantom data from energy[0] to other energy levels

	INTEGER noPointsConv; //number of points used for piecewise linear conversion
	point * c[7];       //list of points for each energy level that are used
						//in piecewise linear conversion

						//BOOLEAN bhc_copied;
	bh_correction()
	{
		bhc = FALSE;
		bhc_poly = FALSE;
		a = NULL;
		for (int i = 0; i < 7; i++)
			c[i] = NULL;
	}

	~bh_correction()
	{
		if (a != NULL)
			delete[] a;
		for (int i = 6; i >= 0; i--)
		{
			if (c[i] != NULL)
				delete[] c[i];
		}
	}

	INTEGER readInputFilePoly();

	INTEGER correction(SnarkPrjFile * BHC_PrjFile);INTEGER idr_correction(
			INTEGER r);

};
#endif	/* _BH_CORRECTION_H */

