/*
***********************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/blob_setparam.cpp $
$LastChangedRevision: 85 $
$Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
$Author: agulati $
***********************************************************

 Given the grid resolution (Blob.delta), this subroutine computes the number of (hexagonal) grids points, so that the picture region of area = GeoPar.nelem * GeoPar.pixsiz is covered by the voronoi neighbourhoods of the grid points. 

 The number of horizontal and vertical grid points of the underlying square grid are respectively Blob.M and Blob.H

 Both Blob.M and Blob.H are assumed to be odd number, so 
 Blob.M = 2*m1+1 and Blob.H = 2*h1 + 1

 The total number of grid points of the underlying square grid is denoted by Blob.area that is equal to Blob.M * Blob.H 

*/
 
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"


void Blob_class::setparam()
{  

  INTEGER m1, h1, mid0, mid1, pr0, pr1;
 

  m1 = (INTEGER)floor(GeoPar.nelem * GeoPar.pixsiz/(Blob.delta));
  h1 = (INTEGER)floor(GeoPar.nelem * GeoPar.pixsiz/(Consts.sqrt3 * Blob.delta));

  pr0  = m1 % 2;
  pr1 = (m1 + 1) % 2;

  mid0  = m1 + 1 - pr0;
  mid1 = m1 + 1 - pr1;
  
  Blob.M = (INTEGER)(2 * m1 + 1);
  Blob.H = (INTEGER)(2 * h1 + 1);
  Blob.diag = (Blob.M + Blob.H) / 2;
  Blob.area = Blob.M * Blob.H; 
  Blob.ljf = 0;
  Blob.ljl = GeoPar.nelem - 1;
  Blob.lkf = 0;
  Blob.lkl = GeoPar.nelem - 1;
  Blob.lhf = (INTEGER)(-1 * (INTEGER)(Blob.H / 2));
  Blob.lhl = (INTEGER)((Blob.H - 1) / 2);
  Blob.lmf = -1 * (INTEGER)(Blob.M / 2);
  Blob.lml = (INTEGER)((Blob.M - 1) / 2);
  Blob.H2 = (INTEGER)(Blob.H / 2);
  Blob.M2 = (INTEGER)(Blob.M / 2);
  Blob.pr = (M2+H2)%2;  
  Blob.sampiblob = samp / Blob.support;
  Blob.di2 = (REAL) 0.5 * Blob.delta; 
  Blob.di2sq3 = Consts.sqrt3 * di2;
  
  fprintf(output, "\n          hexagonal grid: there are %5i rows, the middle row has %5i grid points and the row next to the middle one has %5i grid points", Blob.H, mid0, mid1);
}

