/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bray.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 this subroutine is the WRAY for blobs
 */

#include <cstdio>
#include <cmath>
#include <stdlib.h>

#include "consts.h"
#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "posit.h"
#include "anglst.h"

#include "uiod.h"

void Blob_class::bray(INTEGER np, INTEGER nr, INTEGER* list, REAL* weight, INTEGER* numb, REAL* snorm)
{
	INTEGER i;
	INTEGER a, b;
	INTEGER ac, bc, bc1, ac1;
	INTEGER af, al;
	INTEGER bf, bl;
	INTEGER laf, lal;
	INTEGER lbf, lbl;
	INTEGER h, m;
	INTEGER lmf, lml;
	INTEGER h1, m1;
	INTEGER mcov;
	REAL n;
	INTEGER sign;
	INTEGER M2, H2;

	REAL ax, ay, axid, ayid;
	REAL mx, my;
	REAL theta, sinth, costh;
	REAL prfx, prlx, prfy, prly;

	REAL x, y, yup, ylo;
	REAL ddeltain, ddeltaout;
	REAL dc, dc1;
	REAL ydelta; //xdelta not used

	REAL slope, islope;
	REAL coef, icoef;
	REAL ord, cte, ctes;
	REAL fcosang;
	REAL multf;
	REAL pinc2;
	REAL hex_voronoi;

	// bug 168 - swr - 9/24/05
	if (GeoPar.div)
	{
		fprintf(output,
				"\n         ***** bray is not usable with divergent geometry\n");
		exit(1);
	}

	*numb = 0;

	*snorm = 0;

	hex_voronoi = Consts.sqrt3 / 2.0 * Blob.delta * Blob.delta;

	multf = GeoPar.pinc / (2.0 * Blob.delta); // pinc/2
	pinc2 = GeoPar.pinc / 2.0;

	laf = (INTEGER) (-0.5 * ((int) (Blob.M / 2) + (int) (Blob.H / 2)));
	lal = (INTEGER) (0.5 * ((int) ((Blob.M - 1) / 2) + (int) ((Blob.H - 1) / 2)));

	lbf = -1 * (INTEGER) (Blob.H / 2);
	lbl = (INTEGER) ((Blob.H - 1) / 2);

	lmf = -1 * (INTEGER) (Blob.M / 2);
	lml = (INTEGER) ((Blob.M - 1) / 2);

	prfx = -0.25 * (Blob.M - 1.0);
	prlx = (-1) * prfx;

	prfy = -0.25 * Consts.sqrt3 * (Blob.H - 1.0);
	prly = (-1) * prfy;

	H2 = (INTEGER) (Blob.H / 2);
	M2 = (INTEGER) (Blob.M / 2);
	posit(np, nr, &ax, &ay, &mx, &my);
	axid = ax / Blob.delta;
	ayid = ay / Blob.delta;

	sinth = my;
	costh = mx;
	theta = acos(mx);
	if (my < 0)
	{
		theta = 2 * Consts.pi - theta; //theta is not from getang, but different from ray to ray, by wei 1/2005
	}
	//added 7/19, think of theta as always betw 0 and pi
	if (theta > Consts.pi)
	{
		theta = theta - Consts.pi;
		sinth = -sinth;
		costh = -costh; //never used anyway
		my = -my;
		mx = -mx;
	}

	if (fabs(my) > Consts.zero)
	{  // non-horizontal lines

		islope = mx / my;
		ord = (axid - islope * ayid);
		coef = 0.5 * (Consts.sqrt3 * islope - 1);

		if (theta < Consts.pi / 6 || theta > 2.0 * Consts.pi / 3.0)
		{  // type 1

			cte = multf / my; // for the "side rays"
			slope = my / mx;
			icoef = 1.0 / coef;
			fcosang = (REAL) fabs(cos(theta + Consts.pi / 6.0));

			mcov = (INTEGER) ceil(multf / fcosang);
			ddeltain = Blob.delta * fcosang;
			ddeltaout = Blob.delta * sinth;

			if (theta < Consts.pi / 6)
			{
				sign = 1;
			}
			else
			{
				sign = -1;
			}

			ctes = cte * sign;

			// parameters of the first grid point
			//find optimum af

			x = prfx;
			yup = (x - ord + ctes) * slope;
			ylo = (x - ord - ctes) * slope;

			if (sign == 1)
			{ // + slope

				if (yup < prfy)
				{
					yup = prfy;
					x = ord - ctes + islope * yup;
				}
				ac = (INTEGER) floor(x - yup * Consts.isqrt3);
			}
			else
			{ //sign == -1

				ac = (INTEGER) floor(x - yup * Consts.isqrt3);
				if (ylo > prly)
				{
					ylo = prly;
					x = ord + ctes + islope * ylo;
					ac = (INTEGER) floor(x - ylo * Consts.isqrt3);
				}
			}

			af = MAX0(laf, ac);

			//find optimum al

			x = prlx;
			yup = (x - ord + ctes) * slope;
			ylo = (x - ord - ctes) * slope;

			if (sign == 1)
			{ // + slope

				if (ylo > prly)
				{
					ylo = prly;
					x = ord + ctes + islope * yup;
				}
				ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
			}
			else
			{ //sign == -1

				ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
				if (yup < prfy)
				{
					yup = prfy;
					x = ord - ctes + islope * yup;
					ac = (INTEGER) ceil(x - yup * Consts.isqrt3);
				}
			}

			al = MIN0(lal, ac);

			//done with af and al
			a = af;
			bc = (INTEGER) (floor(icoef * (a - ord)));

			bf = MAX0(bc - mcov, lbf);
			bl = MIN0(bc + mcov, lbl);

			x = a + 0.5 * bc;
			y = 0.5 * Consts.sqrt3 * bc;
			dc = Blob.delta * (REAL) fabs(-my * (axid - x) + mx * (ayid - y));
			bc1 = bc + 1;
			dc1 = ddeltain - dc;

			b = bc;
			n = dc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

			//starting the loops--------------------------------

			while (a <= al)
			{
				if (bf <= bl)
				{
					while (b >= bf)
					{

						if (m >= lmf && m <= lml && b <= lbl)
						{
							if (n < pinc2)
							{

								list[*numb] = h1 * Blob.M + m1;

								*numb = *numb + 1;
							}
						}

						b--;
						m--;
						m1--;
						h1--;
						n += ddeltain;
					}

					b = bc1;
					n = dc1;
					m = 2 * a + b;
					h = b;
					m1 = m + M2;
					h1 = h + H2;

					while (b <= bl)
					{

						if (m >= lmf && m <= lml && b >= lbf)
						{
							if (n < pinc2)
							{

								list[*numb] = h1 * Blob.M + m1;
								*numb = *numb + 1;
							}
						}

						b++;
						m++;
						// h++;
						m1++;
						h1++;
						n += ddeltain;
					}
				} //if bf<= bl

				dc = dc + (REAL) sign * ddeltaout;
				dc1 = ddeltain - dc;

				if (dc > ddeltain || dc < 0.0)
				{
					bc = bc + sign;
					dc = (REAL) sign * (fabs(dc) - ddeltain);
					bc1 = bc + 1;
					dc1 = ddeltain - dc;
					bf = MAX0(bc - mcov, lbf);
					bl = MIN0(bc + mcov, lbl);
				}

				a++;
				b = bc;
				n = dc;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;

			} //while a
		}   //type 1

		else
		{   // type 2
			fcosang = (REAL) fabs(cos(theta + Consts.pi / 6.0));
			mcov = (INTEGER) ceil(multf / sinth);

			// constants independent of the first grid point
			bf = lbf;
			bl = lbl;
			ddeltain = Blob.delta * sinth;
			ddeltaout = Blob.delta * fcosang;

			if (theta < Consts.pi / 3)
			{
				sign = 1;
			}
			else
			{
				sign = -1;
			}

			//----------------------------------------------
			// parameters of the first grid point

			b = bf;

			ac = (INTEGER) (floor(coef * b + ord));

			af = MAX0(ac - mcov, laf);
			al = MIN0(ac + mcov, lal);

			x = ac + 0.5 * b;
			y = 0.5 * Consts.sqrt3 * b;
			dc = Blob.delta * (REAL) fabs(-my * (x - axid) + mx * (y - ayid));

			ac1 = ac + 1;
			dc1 = ddeltain - dc;

			//init values
			a = ac;
			n = dc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

			while (b <= bl)
			{
				if (af <= al)
				{
					while (a >= af)
					{

						if (m >= lmf && m <= lml)
						{

							if (n < pinc2)
							{
								list[*numb] = h1 * Blob.M + m1;

								*numb = *numb + 1;
							}
						}
						a--;
						m -= 2;
						m1 -= 2;
						n += ddeltain;
					}

					a = ac1;
					n = dc1;
					m = 2 * a + b;
					h = b;
					m1 = m + M2;
					h1 = h + H2;

					while (a <= al)
					{

						if (m >= lmf && m <= lml)
						{
							if (n < pinc2)
							{

								list[*numb] = h1 * Blob.M + m1;

								*numb = *numb + 1;
							}
						}
						a++;
						m += 2;
						m1 += 2;
						n += ddeltain;
					}
				} //if af <= al

				dc = dc + (REAL) sign * ddeltaout;
				dc1 = ddeltain - dc;

				if (dc > ddeltain || dc < 0.0)
				{

					ac = ac + sign;
					dc = (REAL) sign * (fabs(dc) - ddeltain);
					ac1 = ac + 1;
					dc1 = ddeltain - dc;
					af = MAX0(ac - mcov, laf);
					al = MIN0(ac + mcov, lal);
				}

				b++;
				a = ac;
				n = dc;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;

			} //while b
		}  // type 2
	}         // if non- horizontal line.

	else
	{

		fcosang = cos(Consts.pi / 6.0);

		mcov = (INTEGER) ceil(multf / fcosang);

		// constants independent of the first grid point
		ddeltain = Blob.delta * fcosang;

		// parameters of the first grid point
		//find optimum af

		x = prfx;
		yup = ayid + multf;
		ylo = yup - 2 * multf;

		ac = (INTEGER) floor(x - yup * Consts.isqrt3);

		af = MAX0(laf, ac);

		//find optimum al

		x = prlx;
		ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
		al = MIN0(lal, ac);

		//done with af and al

		a = af;
		bc = (INTEGER) (floor(2 * ayid * Consts.isqrt3));
		bf = MAX0(bc - mcov, lbf);
		bl = MIN0(bc + mcov, lbl);

		y = 0.5 * Consts.sqrt3 * bc;
		ydelta = y * Blob.delta;
		dc = (REAL) fabs(ydelta - ay);
		bc1 = bc + 1;
		dc1 = ddeltain - dc;
		//nc1 = (INTEGER)(samp_pinc * dc1);

		// init values
		b = bc;
		n = dc;
		m = 2 * a + b;
		h = b;
		m1 = m + M2;
		h1 = h + H2;

		// starting the loops

		while (a <= al)
		{
			if (bf <= bl)
			{
				while (b >= bf)
				{

					if (m >= lmf && m <= lml && b <= lbl)
					{
						if (n < pinc2)
						{

							list[*numb] = h1 * Blob.M + m1;

							*numb = *numb + 1;
						}
					}
					b--;
					m--;
					m1--;
					h1--;
					n += ddeltain;
				}

				b = bc1;
				n = dc1;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;

				while (b <= bl)
				{

					if (m >= lmf && m <= lml && b >= lbf)
					{
						if (n < pinc2)
						{

							list[*numb] = h1 * Blob.M + m1;

							*numb = *numb + 1;
						}
					}
					b++;
					m++;
					m1++;
					h1++;
					n += ddeltain;
				}
			} // if bf <= bl

			a++;
			b = bc;
			//n = nc;
			n = dc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

		} //while a
	}  // fabs (my)==0

	if (*numb != 0)
	{
		for (i = 0; i < *numb; i++)
		{
			weight[i] = hex_voronoi;
		}
	}

	*snorm = hex_voronoi * hex_voronoi * (*numb);
	if (trace > 6)
	{
		fprintf(output,
				"\n          bray    np = %5i  nr = %5i  numb = %5i  snorm = %10.4f",
				np, nr, *numb, *snorm);
	}

	if ((*numb > 0) && (trace > 9))
	{
		for (int i = 0; i < *numb; i++)
		{

			fprintf(output, "\n                  %5i", list[i]);  //bug 139
		}
	}
	return;
}
