/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bsmooth.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 A NINE-POINT WEIGHTED, NON-LINEAR SMOOTHING PROCEDURE.
 FOR DETAILS, SEE THE SNARK MANUAL.
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "consts.h"
#include "uiod.h"
#include "blob.h"

void Blob_class::bsmooth(
REAL* a,      // image
		INTEGER M,    // number of rows
		INTEGER H,    // number of columns
		REAL thresh,  // treshold
		REAL w1,      // weight of center
		REAL w2      // weight of horizontal and diag neighbous
		)
{
	REAL w3 = w2;       // weight of diagonal neighbous
	REAL* row1 = new REAL[M]; // row buffer 1
	REAL* row2 = new REAL[M]; // row buffer 2
	REAL* pres = row2;        // current row before smooth
	REAL* prev = row1;        // previous row before smooth
	INTEGER lmf, lml;
	INTEGER lhf, lhl;
	INTEGER m, h;
	INTEGER m1, h1;
	INTEGER ind;

	BOOLEAN once = TRUE; // message flag

	REAL centre;      // curent grid point value

	REAL smuth;       // weighted sum of neighbours
	REAL divide;      // sum of weights
	REAL value;       // neighbour value

	lhf = (INTEGER) (-1 * (INTEGER) (H / 2));
	lhl = (INTEGER) ((H - 1) / 2);

	lmf = -1 * (INTEGER) (M / 2);
	lml = (INTEGER) ((M - 1) / 2);

	for (h = lhl; h >= lhf; h--)
	{
		REAL* itemp = prev;  // swap the buffers (necessary?)
		prev = pres;
		pres = itemp;

		for (m = lmf; m <= lml; m++)
		{ // m and h must have the same parity!!
			if ((m + h) % 2 == 0)
			{
				m1 = m + Blob.M2;
				h1 = h + Blob.H2;
				ind = h1 * M + m1;

				centre = a[ind];
				pres[m1] = centre;
				smuth = w1 * centre;
				divide = w1;

				if (h != lhl)
				{                   // if not first row
					if (m != lmf)
					{                 // if not first column
						value = prev[m1 - 1];       // prev row prev column  #1

						if ((REAL) fabs(value - centre) <= thresh)
						{
							smuth += w3 * value;
							divide += w3;
						}
					}

					if (m != lml)
					{              // if not last column
						value = prev[m1 + 1];      // prev row, next column   #2

						if ((REAL) fabs(value - centre) <= thresh)
						{
							smuth += w3 * value;
							divide += w3;
						}
					}
				}

				if (m >= lmf + 2)
				{                   // if not first or second column
					value = pres[m1 - 2];        // same row, two prev column #3

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w2 * value;
						divide += w2;
					}
				}

				if (m <= lml - 2)
				{        // if not last column or one before last one
					value = a[ind + 2];          // same row, two next column #4

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w2 * value;
						divide += w2;
					}
				}

				if (h != lhf)
				{             // if not last row
					if (m != lmf)
					{                 // if not first column
						value = a[ind - M - 1];     // next row prev column   #5

						if ((REAL) fabs(value - centre) <= thresh)
						{
							smuth += w3 * value;
							divide += w3;
						}
					}

					if (m != lml)
					{           // if not last column
						value = a[ind - M + 1];      // next row next column  #6

						if ((REAL) fabs(value - centre) <= thresh)
						{
							smuth += w3 * value;
							divide += w3;
						}
					}
				}

				if ((REAL) fabs(divide) > Consts.zero)
				{
					a[ind] = smuth / divide;
				}
				else
				{
					if (once)
					{
						fprintf(output, "\n *** warning - sum of weights=zero");
						once = FALSE;
					}

					a[ind] =
							((smuth - Consts.infin) > 0) ? (REAL) 1.0 :
							(smuth == Consts.infin) ? (REAL) 0.0 : (REAL) -1.0;
				}

				//l+=2;    //blob grid
			}
		}
	}

	delete[] row1;  // bug 92 - Lajos - 03/02/2005
	delete[] row2;  // bug 92 - Lajos - 03/02/2005

	return;
}
