/*
**********************************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/dcon_dconbl.cpp $
$LastChangedRevision: 85 $
$Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
$Author: agulati $
**********************************************************************

 CONVOLUTION-TYPE METHOD FOR DIVERGENT DATA WITH BAND-LIMITING
 FILTER.
     BASED ON
        CONVOLUTION RECONSTRUCTION TECHNIQUES FOR DIVERGENT BEAMS
        G. T. HERMAN, A. V. LAKSHMINARAYANAN AND A. NAPARSTEK
        COMPUTERS IN BIOLOGY AND MEDICINE
        TO APPEAR
     WRITTEN BY G. T. HERMAN, MARCH 1975
     REVISED FOR THE CDC3500, MARCH 1976
     REVISED FOR SNARK77 BY T. CHANG, MAY 1977
 ***********************************************************
*/

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "qintp.h"
#include "second.h"
#include "projfile.h"

#include "dcon.h"

#ifdef FFCOMPARE

REAL Sin(REAL x)
{
  REAL sum = 0;

  REAL y = x;
  REAL z = 1;
  REAL a = 1;

  for(int i = 1; i < 10; i++) {
    if((i % 2) == 0) {
      sum -= z/a;
    }
    else {
      sum += z/a;
    }

    a *= (2 * i + 1) * 2 * i; // !
    z *= x * x; // x ^ (2 *  i)
  }

  return x * sum;
}

#else

#define Sin(X) sin((X))

#endif

void dcon_class::dconbl(REAL* recon, INTEGER interp, INTEGER missp, INTEGER nconr, REAL wconv, INTEGER method)
{      
////////////////////////////////////////////////////////

  INTEGER limup;
  REAL wconw;
  REAL* j = NULL;
  INTEGER mdrym1;
  REAL* mq = NULL;
  REAL alfa;
  REAL alfasq;
  REAL a;
  int i;
  int m;
  int n;

  INTEGER nraym1;
  REAL concen;
  INTEGER limmid;

  REAL halfl;
  REAL twodr;
  REAL fmidry;
  INTEGER lastpr;
  REAL beta0;
  REAL sinth;
  REAL costh;
  REAL theta;
  REAL stime;
  REAL ctime;
  REAL btime;
  REAL bend;
  int np;
  REAL dtam;
  REAL dtar;
  int nr;
  REAL dtal;
  REAL send;
  INTEGER mqm;
  int nm;
  INTEGER inp;

  INTEGER inm;
  INTEGER inr;
  REAL pr;
  INTEGER inl;
  REAL pl;
  int mm;
  REAL cend;
  REAL cbeta;
  REAL sbeta;
  REAL beta1;
  REAL w;
  REAL rcosi;
  REAL rcosy;
  REAL rcosx;
  REAL rsini;
  REAL rsiny;
  REAL rsinx;
  REAL wtwodr;
  REAL wjrfi;
  REAL wjrfx;
  REAL wjrfy;
  REAL anginy;
  REAL angiin;
  REAL fkjrfx;
  REAL subtr;
  REAL sob;
  REAL cob;
  REAL wrsq;
  REAL prini;
  INTEGER ind;
  REAL rcos;
  REAL rsin;
  REAL wjrf;
  REAL fract;
  REAL fkjrf;
  REAL* mod;
  int k;
  REAL usq;
  REAL frynum;
  REAL p;

////////////////////////////////////////////////////////
  REAL* g;

  // ASSIGN SPACE FOR SMOOTHED PROJECTION DATA, ECHO SMOOTHING WEIGHTS

  g = new REAL[GeoPar.nrays];
  limup = GeoPar.nrays - 1; // index of last ray

  if(!((wconv > 0.0) && (wconv <= 1.0))) wconv = (REAL) 1.0;
  wconw = ((REAL) 1.0 - wconv) / (REAL) 2.0;

  // ALLOCATE SPACE FOR CONVOLUTION INFORMATION

  j = new REAL[GeoPar.nrays];

  mdrym1 = GeoPar.midray - 1;

  //mq = new REAL[mdrym1];
  mq = new REAL[mdrym1 + 1];  //(JD 1/20/04)


  // WEIGHT ARRAY J(I)

  alfa = GeoPar.pinc / GeoPar.stod;
  alfasq = alfa * alfa;
  a = alfa * GeoPar.radius;
  j[GeoPar.midray] = a;

  for(i = 1; i <= GeoPar.midray; i++) 
  {
    if(GeoPar.tang) j[GeoPar.midray + i] = a / (REAL) sqrt((REAL) 1.0 + alfasq * SQR(((REAL)(i))));
    if(GeoPar.arc) j[GeoPar.midray + i]  = a * (REAL) cos( ((REAL)(i)) * alfa);
    j[GeoPar.midray - i] = j[GeoPar.midray + i];
  }

  ////////////////////////////////////////////////////
#ifdef FFCOMPARE
  fprintf(output,"\n***** test data 1 *****************************");
  for(i = 0; i < GeoPar.nrays; i++) 
  {
    if((i % 3) == 0) fprintf(output,"\n");
    fprintf(output," %36.30f", j[i]);
  }
  fprintf(output,"\n");
#endif
  ////////////////////////////////////////////////////


  // CONVOLUTING FUNCTION Q(M)

  for(m = 0; m < GeoPar.midray; m++) 
  {
    n = 2 * m + 1; // 1, 3, 5, ...
    if(GeoPar.tang) mq[m] = (REAL) -1.0 / ((REAL) 2.0 * SQR(Consts.pi * n * a));
#ifndef FFCOMPARE
    if(GeoPar.arc) mq[m] = (REAL) -1.0 / ((REAL) 2.0 * SQR(Consts.pi * (REAL) sin((REAL) (n * alfa))));

#else
    REAL xxx = n * alfa;
    REAL yyy = Sin(xxx);
    REAL zzz = sin(xxx);

    if(GeoPar.arc) mq[m] = (REAL) -1.0 / ((REAL) 2.0 * SQR(Consts.pi * yyy));

//    fprintf(output,"\n %36.30f %36.30f %36.30f", (double) mq[m], (double) xxx, (double) alfa);
    fprintf(output,"\n %36.30f", (double) mq[m]);
#endif
  }

  ////////////////////////////////////////////////////
#ifdef FFCOMPARE
  fprintf(output,"\n***** test data 2*****************************");
  for(m = 0; m < GeoPar.midray; m++) 
  {
    if((m % 3) == 0) fprintf(output,"\n");
    fprintf(output," %36.30f", mq[m]);
  }
  fprintf(output,"\n");
#endif
  ////////////////////////////////////////////////////



  // CONSTANTS NEEDED FOR CONVOLUTION AND BACKPROJECTION

  nraym1 = GeoPar.nrays - 1;
  if(GeoPar.tang) concen = (REAL) 1.0 / ((REAL) 8.0 * SQR(a));
  if(GeoPar.arc) concen = (REAL) 1.0 / ((REAL) 8.0 * SQR(alfa));
  limmid = MIN0(nconr, mdrym1);
  mod = new REAL[GeoPar.nrays];

  ///midmod = mod + midray;
  halfl = ((REAL)(GeoPar.midpix)) * GeoPar.pixsiz;
  twodr = (REAL) 2.0 / GeoPar.radius;
  fmidry = (REAL) GeoPar.midray + 1;

  // INITIALIZE BETA0 TO BE USED IN CALCULATING WEIGHTS FOR
  // BETA INTEGRATION

  lastpr = ((GeoPar.prjnum - 1) / missp) * missp;
  Anglst.getang(lastpr, &beta0, &sinth, &costh);
  Anglst.getang(0, &theta, &sinth, &costh);


  // INITIALIZE TIMERS

  stime = 0.0;
  ctime = 0.0;
  btime = 0.0;
  second(&bend);


  // THIS IS WHERE THE RECONSTRUCTION STARTS
  // BUILD UP RECONSTRUCTED PICTURE ONE PROJECTION AT A TIME

  for(np = 0; np < GeoPar.prjnum; np += missp) 
  {

    ProjFile.ReadProj(np, g, GeoPar.nrays);
    
    // debug ////////////////////////////////////////////////////
    // input verification
#ifdef FFCOMPARE
    fprintf(output,"\n**********************************");
    for(nr = 0; nr < GeoPar.nrays; nr++) 
	{
      if((nr % 3) == 0) fprintf(output,"\n");
      fprintf(output," %36.30f", g[nr]);
    }
#endif
    //////////////////////////////////////////////////////


    dtam = g[0];
    dtar = g[1];
    g[0] = (wconv * dtam + wconw * dtar) * j[0];

    for(nr = 1; nr < nraym1; nr++) 
	{
      dtal = dtam;
      dtam = dtar;
      dtar = g[nr + 1];
      g[nr] = (wconv * dtam + wconw * (dtar + dtal)) * j[nr];
    }

    dtal = dtam;
    dtam = dtar;
    g[nraym1] = (wconv * dtam + wconw * dtal) * j[nraym1];

    second(&send);
    stime += send - bend;


#ifdef FFCOMPARE
    fprintf(output,"\n***** test data 3 *****************************");
    for(nr = 0; nr < nraym1; nr++) {
      if((nr % 3) == 0) fprintf(output,"\n");
      fprintf(output," %36.30f", g[nr]);
    }
    fprintf(output,"\n");
#endif


    // COMPUTE CONVOLUTED PROJECTION DATA

    p = concen * g[GeoPar.midray];

    mqm = 0;
    for(nm = 1; nm <= limmid; nm += 2) {
      p += (g[GeoPar.midray + nm] + g[GeoPar.midray - nm]) * mq[mqm];
      mqm++;
    }

    mod[GeoPar.midray] = p;

    // WE HAVE NOW TAKEN CARE OF THE CENTER RAY

    for(m = 1; m <= GeoPar.midray; m++) {
      inr = GeoPar.midray + m;
      pr = concen * g[inr];
      inl = GeoPar.midray - m;
      pl = concen * g[inl];

      if(!(nconr == 0)) {

        mqm = 0;

        for(nm = 1; nm <= nconr; nm += 2) {
          inp = inr + nm;
          if(inp > limup) {
            for(mm = nm; mm <= nconr; mm += 2) {
              inm = inr - mm;
              if(inm < 0) break;
              pr += g[inm] * mq[mqm];
              inp = inl + mm;
              pl += g[inp] * mq[mqm];
              mqm++;
            }
            break;
          }

          inm = inr - nm;

          pr += ( g[inp] + g[inm] ) * mq[mqm];
          inp = inl + nm;
          inm = inl - nm;
          pl += ( g[inp] + g[inm] ) * mq[mqm];



          mqm++;
        }
      }

      mod[GeoPar.midray + m] = pr;
      mod[GeoPar.midray - m] = pl;

    }

#ifdef FFCOMPARE

  fprintf(output,"\n***** test data mod *****************************");
  for(m = 0; m < GeoPar.nrays; m++) {
    if((m % 3) == 0) fprintf(output,"\n");
    fprintf(output," %36.30f", mod[m]);
  }
  fprintf(output,"\n");

#endif
  ////////////////////////////////////////////////////
  

    second(&cend);
    ctime += cend - send;

    // BACK-PROJECT CONVOLUTED PROJECTION DATA

    cbeta = sinth;
    sbeta = -costh;

    // CALCULATE WEIGHTS FOR BETA INTEGRATION

    beta1 = theta;
    if(np >= lastpr) Anglst.getang(0, &theta, &sinth, &costh);
    if(np < lastpr) Anglst.getang(np+missp, &theta, &sinth, &costh);

    w = (REAL) fmod(Consts.twopi + theta - beta0, Consts.twopi) / (REAL) 2.0;

    beta0 = beta1;

    // CALCULATE BACK-PROJECTION INFORMATION

    rcosi = halfl * (sbeta - cbeta);
    rcosy = - GeoPar.pixsiz * sbeta;
    rcosx = GeoPar.pixsiz * cbeta;
    rsini = -halfl * (sbeta + cbeta);
    rsiny = rcosx;
    rsinx = -rcosy;
    if(GeoPar.arc) w /= SQR(GeoPar.radius);
    wtwodr = w * twodr;
    wjrfi = w - wtwodr * rsini;
    wjrfx = -wtwodr * rsinx;
    wjrfy = -wtwodr * rsiny;
    anginy = (SQR(GeoPar.pixsiz / GeoPar.radius)) / alfa;
    angiin = ((REAL)(GeoPar.midpix)) * anginy;
    fkjrfx = rcosx / a - angiin;
    subtr = fkjrfx * GeoPar.midpix;
    sob = halfl * sbeta;
    cob = GeoPar.radius - halfl * cbeta;
    wrsq = w * SQR(GeoPar.radius);
    prini = GeoPar.radius + rsini;

    // BACK-PROJECTION

    ind = 0;
    for(i = 0; i < GeoPar.nelem; i++) 
	{
		if(method >= 0) 
		{
			rcosi += rcosy;
			rcos = rcosi;
			prini += rsiny;
			rsin = prini;
		}
		if(method != 0) 
		{

			wjrfi += wjrfy;
			wjrf = wjrfi;
			if(method <= 0) {
				sob += rcosy;
				cob += rsiny;
				fract = sob / cob;
				fkjrfx += anginy;
				subtr += angiin;
				if(GeoPar.arc) fract = (REAL) atan(fract);
				fkjrf = fract / alfa - subtr;
			}
		}

		for(k = 0; k < GeoPar.nelem; k++) 
		{
			if(method >= 0) 
			{
				rcos += rcosx;
				rsin += rsinx;

				if(method <= 0) 
				{
					usq = SQR(rsin);
					if(GeoPar.arc) usq += SQR(rcos);
					wjrf = wrsq / usq;
				}
				fract = rcos / rsin;
				if(GeoPar.arc) fract = (REAL) atan(fract);
				fkjrf = fract / alfa;
			}
			if(method != 0) 
			{
				wjrf += wjrfx;
				if(method < 0) fkjrf += fkjrfx;
			}
			frynum = fmidry - fkjrf;


#ifdef FFCOMPARE
			fprintf(output,"\n frynum = %36.30f wjrf = %36.30f", frynum, wjrf);
#endif


			recon[ind] += wjrf * qintp(frynum, mod, GeoPar.nrays, interp);


			ind++;
		}
    }

    second(&bend);
    btime += bend - cend;
  }
  
  fprintf(output, "\n          total time for obtaining smoothed projection data was %9.3f", stime);   // changed precision to two digits - swr 1/21/06
  fprintf(output, "\n          total time for convoluting projection data was %16.3f", ctime);         // ditto
  fprintf(output, "\n          total time for back-projecting was %28.3f", btime);                     // ditto

  
  if(j != NULL) delete[] j;
  if(mq != NULL) delete[] mq ;           //wei 3/2005
  
  delete[] mod;  // bug 92 - Lajos - 03/02/2005
  delete[] g;  // bug 92 - Lajos - 03/02/2005
  return;
}      
