/*
 **********************************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/effpick.cpp $
 $LastChangedRevision: 79 $
 $Date: 2014-07-01 15:33:39 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 **********************************************************************
 */

#include <cstdio>
#include <cmath>

#include "uiod.h"
#include "blkdta.h"
#include "primfc.h"
#include "effpick.h"

void factorization(INTEGER num, INTEGER** decomp, INTEGER *num_dig)
{
	// factorizes num and put the factors into vector decomp of size num_dig

	INTEGER m2s, nipf, mnipf, k;

	INTEGER* ipf = NULL;


	if (num == 1)
	{  //bug 144 earlier version gets *num_dig=0 when num == 1
		*num_dig = 1;
		*decomp = new INTEGER[1];
		*decomp[0] = 1;
		return;
	}

	else
	{ //num > 1

		mnipf = (INTEGER) ceil(log((REAL) num) / log(2.0));

		ipf = new INTEGER[mnipf];
		primfc(num, &m2s, &nipf, ipf, mnipf);

		*num_dig = m2s + nipf;
		*decomp = new INTEGER[*num_dig];

		for (k = 0; k < m2s; k++)
		{
			(*decomp)[k] = 2;
		}

		for (k = 0; k < nipf; k++)
		{
			(*decomp)[k + m2s] = ipf[k];
		}

		delete[] ipf;  //wei 3/2005
		return;
	}
}

void tau_map(INTEGER num_dig, INTEGER* decomp, INTEGER* old_num,
		INTEGER* new_num)
{
	// see the definition of the tau mapping in "Herman and Meyer. Algebraic reconstruction techniques can be made computationally efficient. IEEE Trans. on Medical Imaging 12: 600--609. 1993"
	static INTEGER cont, ind;

	if (old_num[0] == -1)
	{
		for (cont = 0; cont < num_dig; cont++)
		{
			old_num[cont] = 0;
			new_num[cont] = 0;
		}
		return;
	}

	for (cont = 0; cont < num_dig; cont++)
	{
		if (old_num[cont] != decomp[cont] - 1)
		{
			ind = cont;
			break;
		}
	}

	for (cont = 0; cont < num_dig; cont++)
	{
		if (cont < ind)
		{
			new_num[cont] = 0;
		}
		else if (cont == ind)
		{
			new_num[cont] = old_num[cont] + 1;
		}
		else
		{
			new_num[cont] = old_num[cont];
		}
	}

	for (cont = 0; cont < num_dig; cont++)
		old_num[cont] = new_num[cont];
}

void get_base(INTEGER* decomp, INTEGER num_dig, INTEGER** base)
{
	// given the factorization vector decomp (of size num_dig) in increasing order, the routine returns another vector whose components are the accumulative product of decomp in the reversed order
	INTEGER cont, aux;

	aux = 1;

	*base = new INTEGER[num_dig];

	(*base)[num_dig - 1] = 1;

	for (cont = 0; cont < num_dig - 1; cont++)
	{
		aux = aux * decomp[num_dig - cont - 1];
		(*base)[num_dig - cont - 2] = aux;
	}

	return;

}

void nu_map(INTEGER num_dig, INTEGER* base, INTEGER* num, INTEGER* result)
{
	// given the the vector num whose components are the digits of a number written in base "base", this routines returns that number as "result"
	INTEGER cont;

	*result = 0;

	for (cont = 0; cont < num_dig; cont++)
	{
		*result = *result + base[cont] * num[cont];
	}

}

void pi_map(INTEGER num_dig, INTEGER* decomp, INTEGER* base, INTEGER** old_num,
		INTEGER** new_num, INTEGER* res)
{
	// given the factorizaton vector decomp and the vector base, this routines is the composition of nu_map and tau_map

	tau_map(num_dig, decomp, (*old_num), (*new_num));
	nu_map(num_dig, base, (*new_num), res);


	return;
}

