/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/ftodd.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THE SUBROUTINES FFT2P , FTODD , UNKPS ARE FAST FOUTIER ROUTINES.
 THE SUBROUTINE SNFFT ,DESCRIBED IN THE SNARK05 MANUAL , MAKES
 USE OF THESE SUBROUTINES.
 THE SUBROUTINES HAVE BEEN TAKEN FROM
 POLGE,R.J., AND BHAGAVAN,B.K., EFFICIENT FAST FOURIER TRANSFORM
 PROGRAMS FOR ARBITRARY FACTORS WITH ONE STEP LOOP UNSCRAMBLING,
 IEEE TRANSACTIONS ON COMPUTERS, MAY 1976.
 */

#include <cstdio>
#include <cmath>    

#include "blkdta.h"

#include "ftodd.h"

void ftodd(
INTEGER imax,
INTEGER istep,
REAL* xs,
REAL* x,
INTEGER n,
INTEGER* nk,
REAL* v,
REAL tpis,
INTEGER nipf,
INTEGER* ipf,
INTEGER mc,
INTEGER k4m4,
INTEGER nipfx,
INTEGER* ipfx,
INTEGER* ipu,
INTEGER* ix,
INTEGER* nfixp,
REAL* vspt,
REAL* t,
BOOLEAN virt)
{
	REAL vs[2];

	INTEGER i;
	INTEGER ip;
	INTEGER sk;
	REAL fix;
	INTEGER kt;
	REAL temp;
	INTEGER kss;
	INTEGER ks;
	INTEGER kp;
	INTEGER kgp;
	INTEGER ktas;
	INTEGER kta;
	INTEGER ia;
	INTEGER l;
	INTEGER ind2;
	INTEGER ind;
	INTEGER indx;
	INTEGER kpm1;
	INTEGER jt;
	INTEGER ktip;
	INTEGER k;
	INTEGER jlim;
	INTEGER kradx;
	INTEGER kradxm;
	INTEGER j;
	INTEGER ir;
	INTEGER inc;
	INTEGER jl;
	INTEGER irm1;
	INTEGER kiu;
	INTEGER kix;
	INTEGER inp;
	INTEGER in;

	for (i = 0; i < nipf; i++)
	{
		ip = ipf[i];
		sk = *nk / ip;
		CA(t, 0, 0)= 1.0;
		CA(t, 1, 0)= 0.0;

		fix = tpis / ip;

		vs[0] = (REAL) cos(fix);
		vs[1] = (REAL) sin(fix);

		CA(vspt, 0, 0)= 1.0;
		CA(vspt, 1, 0)= 0.;

		for (kt = 1; kt < ip; kt++)
		{
			temp = CA(vspt, 0, kt-1)* vs[0] - CA(vspt, 1, kt-1) * vs[1];
			CA(vspt, 1, kt) = CA(vspt, 0, kt-1) * vs[1] + CA(vspt, 1, kt-1) * vs[0];
			CA(vspt, 0, kt) = temp;
		}

		vs[0] = 1.;
		vs[1] = 0.;

		for (kss = 0; kss < sk; kss++)
		{

			ks = kss;

			for (kp = 1; kp < ip; kp++)
			{
				temp = CA(t, 0, kp-1)* vs[0] - CA(t, 1, kp-1) * vs[1];
				CA(t, 1, kp) = CA(t, 0, kp-1) * vs[1] + CA(t, 1, kp-1) * vs[0];
				CA(t, 0, kp) = temp;
			}

			for (kgp = 0; kgp < n; kgp += *nk)
			{
				ktas = ks + kgp;
				kta = ktas;

				for (kt = 0; kt < ip; kt++)
				{
					ia = kta * istep;

					for (l = 0; l < imax; l++)
					{
						ind2 = l + ia;
						ind = l + kt * istep;
						CA(xs, 0, ind)= CA(x, 0, ind2);
						CA(xs, 1, ind)= CA(x, 1, ind2);
					}
					kta += sk;
				}

				ia = ktas * istep;

				for (l = 0; l < imax; l++)
				{
					ind = l + ia;
					CA(x, 0, ind)= CA(xs, 0, l);
					CA(x, 1, ind)= CA(xs, 1, l);
				}

				for (kp = 1; kp < ip; kp++)
				{
					for (l = 0; l < imax; l++)
					{
						ind = l + kp * istep;
						indx = l + ia;
						CA(x, 0, indx)+= CA(xs, 0, ind);
						CA(x, 1, indx)+= CA(xs, 1, ind);
					}
				}

				kta = ktas;
				for (kp = 1; kp < ip; kp++)
				{
					kta += sk;
					ia = kta * istep;

					for (l = 0; l < imax; l++)
					{
						ind = l + ia;
						CA(x, 0, ind)= CA(xs, 0, l);
						CA(x, 1, ind)= CA(xs, 1, l);
					}

					kpm1 = kp;
					jt = kpm1;

					for (kt = 1; kt < ip; kt++)
					{
						jt = jt - ip * (jt / ip); // jt %= ip // ??? mk
						for (l = 0; l < imax; l++)
						{
							ind = l + kt * istep;
							indx = l + ia;
							CA(x, 0, indx)+= CA(vspt, 0, jt) * CA(xs, 0, ind) - CA(vspt, 1, jt) * CA(xs, 1, ind);
							CA(x, 1, indx)+= CA(vspt, 0, jt) * CA(xs, 1, ind) + CA(vspt, 1, jt) * CA(xs, 0, ind);
						}
						jt += kpm1;
					}

					for (l = 0; l < imax; l++)
					{
						indx = l + ia;
						temp = CA(x, 0, indx)* CA(t, 0, kp) - CA(x, 1, indx) * CA(t, 1, kp);
						CA(x, 1, indx)= CA(x, 0, indx) * CA(t, 1, kp) + CA(x, 1, indx) * CA(t, 0, kp);
						CA(x, 0, indx)= temp;
					}
				}
			}
			temp = v[0] * vs[0] - v[1] * vs[1];
			vs[1] = v[0] * vs[1] + v[1] * vs[0];
			vs[0] = temp;
		}
		*nk = sk;

		CA(xs, 0, 0)= v[0];
		CA(xs, 1, 0)= v[1];

		for (l = 1; l < ip; l++)
		{
			temp = CA(xs, 0, 0)* v[0] - CA(xs, 1, 0) * v[1];
			CA(xs, 1, 0) = CA(xs, 0, 0) * v[1] + CA(xs, 1, 0) * v[0];
			CA(xs, 0, 0) = temp;
		}

		v[0] = CA(xs, 0, 0);
		v[1] = CA(xs, 1, 0);
	}

	if (nipfx < 2)
		return;

	ktip = mc - mc / 2;

	ipfx[0] = 4;
	if (mc == 1)
		ipfx[0] = 2;
	if (ktip == 2)
		ipfx[1] = 2;

	for (k = 0; k < nipf; k++)
	{
		ind = k + ktip;
		ipfx[ind] = ipf[k];
	}

	ix[0] = 0;
	jlim = 1;

	for (i = 0; i < nipfx; i++)
	{
		kradx = ipfx[i];
		kradxm = kradx - 1;

		for (j = 0; j < jlim; j++)
		{
			ir = j;
			inc = 0;
			ix[j] *= kradx;

			for (jl = 0; jl < kradxm; jl++)
			{
				ir += jlim;
				inc++;
				ix[ir] = ix[j] + inc;
			}
		}
		jlim *= kradx;
	}

	irm1 = ir;
	kiu = 0;

	for (kix = 1; kix < irm1; kix++)
	{
		if (!(ix[kix] != kix))
		{
			(*nfixp)++;
			ipu[kiu] = ix[kix] * k4m4;
			ix[kix] = -ix[kix];
			kiu++;
		}
	}

	for (kix = 1; kix < irm1; kix++)
	{
		if (!(ix[kix] < 0))
		{
			inp = kix;
			do
			{
				in = ix[inp];
				ix[inp] = -ix[inp];
				inp = in;
				ipu[kiu] = in * k4m4;

				kiu++;

			} while (inp != kix);

			ipu[kiu] = 0;
			kiu++;
		}
	}

	ipu[kiu] = n;
	return;
}
