/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/get_seed.c $
 $LastChangedRevision: 118 $
 $Date: 2014-07-09 14:22:29 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 *
 * Feb 13, 2011 Joanna Klukowska
 *     This file is no longer user. A single seed for snark14 experimenter run
 *     is provided in the input file, or based on time (if the input file does not
 *     contain it.
 *
 */

#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "get_seed.h"

/*---------------------------- get_seed.c ---------------------------------
 This function checks the file specified on line 3 of the
 SuperSNARK input sequence and obtaines the seed value (if it is
 present).

 INPUTS:
 proj_flname - name of file containing information about generation of the
 projection data. This is the file specified on line 3 of the
 SuperSNARK input sequence.
 seed - seed generated by current time.

 OUTPUTS:
 seed - seed to be used in random number generator.
 ----------------------------------------------------------------------------*/

void get_seed(char* proj_flname, long* seed)
{
	char string[MAXLINESIZE], line[MAXLINESIZE];
	FILE *projfl;
	long inseed;

	if ((projfl = fopen(proj_flname, "r")) == NULL)
	{
		errorc("error in opening file", proj_flname);
	}

	while ((fgets(string, sizeof(string), projfl)) != NULL)
	{
		if (strncasecmp(string, "seed", 4) == 0)
		{
			/* if string has 'seed' then ... */
			if (sscanf(string, "%s %ld", line, &inseed) == 2)
			{
				if (inseed >= 0)
					*seed = inseed;
				//else if inseed < 0  use the current time as seed
			}
			else
			{
				//if number not specified, the default = 0
				*seed = 0;
			}
		}
		else
		{
			;
		}
	}

	fclose(projfl);
}
