/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/getiters.c $
 $LastChangedRevision: 86 $
 $Date: 2014-07-02 16:45:09 -0400 (Wed, 02 Jul 2014) $
 $Author: olangthaler $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>
#include <strings.h>
#include <malloc.h>
#include <stdio.h>
#include "experimenter.h"
#include "errorc.h"
#include "getiters.h"

/* ------------------------- getiters.c -------------------------------

 This function reads the iteration numbers from the file containing the comparisons to be made.

 INPUTS:
 anafl - file pointer to the file containing the comparisons to be made

 OUTPUTS:
 itr1 - array of length numiters containing the iteration numbers for the first algorithm.
 itr2 - array of length numiters containing the iteration numbers for the first algorithm.
 numiters - number of iterations to be compared.
 string - line of text after the set of iteration numbers just read; this line will be COMPARE, MODE, or END commands.
 */

void getiters(FILE* anafl, int** itr1, int** itr2, int* numiters, char* string)
{
	int count = 0, iter1, iter2;
	char line[MAXLINESIZE];

	/* allocate memory for first set of iteration numbers */

	*itr1 = (int *) malloc(sizeof(int));
	*itr2 = (int *) malloc(sizeof(int));
	if ((*itr1 == NULL) || (*itr2 == NULL))
	{
		errorc("in analyze.c: memory allocation failed in ", "getiters");
	}

	/* at this point the file pointer anafl is at the first set of iteration
	 numbers since the previous line read in by the calling program was a COMPARE
	 line */

	while (1)
	{
		fgets(line, sizeof(line), anafl);
		while (isSkip(line))     //jump over comment lines and blank lines
		{
			fgets(line, sizeof(line), anafl);
		}
		if (sscanf(line, "%d %d", &iter1, &iter2) == 2)
		{
			if (iter1 == 0)	(*itr1)[count] = 0; //??? whatever the last iteration is;
			if (iter2 == 0) (*itr2)[count] = 0; //??? whatever the last iteration is;

			(*itr1)[count] = iter1; /* store iteration numbers */
			(*itr2)[count] = iter2;
			count++; /* number of iteration lines read in */
			*itr1 = (int *) realloc(*itr1, (count + 1) * sizeof(int));
			*itr2 = (int *) realloc(*itr2, (count + 1) * sizeof(int));
			if ((*itr1 == NULL) || (*itr2 == NULL))
			{
				errorc("in analyze.c: memory reallocation failed in ","getiters");
			}
		}
		else
		{
			break; /* break out of loop when iteration numbers done */
		}
	}
	*numiters = count;
	// skip FLAG line
	if (strncasecmp(line, "flag", 4) == 0)
	{
		fgets(line, sizeof(line), anafl);
	}
	strcpy(string, line);
	/* line and string now contain the string of text after
	 the set of iteration numbers and the FLAG command just read in.
	 This string will be COMPARE, MODE or END.
	 If not then terminate program */
	if ((strncasecmp(string, "mode", 4) != 0) && (strncasecmp(string, "comp", 4) != 0) && (strncasecmp(string, "end", 3) != 0))
	{
		errorc("In getiters: error in comparison file at line ", string);
	}
}
