/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/inputfile.cpp $
 $LastChangedRevision: 86 $
 $Date: 2014-07-02 16:45:09 -0400 (Wed, 02 Jul 2014) $
 $Author: olangthaler $
 ***********************************************************

 THE FIRST 7 ROWS OF 'OBJECT' STORE THE DENSITY FOR THE 7 ENERGY
 LEVELS, 8TH  STORES THE AVERAGE DENSITY;
 9TH ROW SPECIFIES TYPE OF OBJECT, 1 FOR ELLIPSE/CIRCLE
 2 FOR RECTANGLE/SQUARE
 3 FOR TRIANGLE
 4 FOR SEGMENT OF CIRCLE
 5 FOR SECTOR OF CIRCLE
 10TH ROW  CX  X-COORDINATE OF CENTER
 11TH ROW  CY  Y-COORDINATE OF CENTER
 12TH ROW  U  SEMI X-AXIS LENGTH
 13TH ROW  V  SEMI Y-AXIS LENGTH
 14TH ROW  ANG  ANGLE OF ROTATION FROM X-AXIS
 15TH ROW  SINANG  SIN OF ANGLE OF ROTATION
 16TH ROW  COSANG  COSINE OF ANGLE OF ROTATION
 17TH ROW RAYL   LENGTH OF RAY INTERSECTED BY OBJECT
 THIS VALUE OF SUBSCRIPT REFERRED TO IN CREAPR ONLY
 */

#include <cstdlib>       
#include <cstdio>
#include <cmath>
#include <time.h>

#include "blkdta.h"
#include "creacm.h"
#include "geom.h"
#include "spctrm.h"
#include "noise.h"
#include "objects.h"
#include "consts.h"
#include "uiod.h"

#include "int2str.h"
#include "anglst.h"
#include "creaer.h"
#include "DIGRand.h"

#include "inputfile.h"

//jk 11/18/2007 introducing variability
REAL variability;
//jk 02/06/09 bug 251
//long newseed;    //the variable Creacm.varseed is used instead
//the array stores values of variability added to each pixel of the phantom
//for each energy level
REAL * variab_array[7];
//the flag is set if the value of variability in the input file is other than zero
BOOLEAN variab_set;
//saves the value of the origical scale which is the mean value for variability calculations
REAL mean;

REAL * phantom_poly_array[7];

void InputFile_class::Open()  //open input file (stdin) and 
{
	infile = stdin;
	if (infile == NULL)
	{
		fprintf(output, "\n **** unable to open input file");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	}
}

void InputFile_class::GetName()	// get name
{
	int i;
	signed CHAR ch;  // changed "CHAR" to "signed CHAR". lajos, Nov 18, 2004

	fprintf(output, "\n");

	//assign the name of reconstruction to the 'name' member of 
	//Creacm (defined in global scope in creacm.cpp	
	for (i = 0; i < 80; i++)
	{  // changed "i < 81" to "i < 80". Lajos, Nov 18, 2004
		//if there are fewer than 80 characters, stop earlier
		if (((ch = fgetc(infile)) == '\n') || (ch == EOF))
		{
			break;
		}
		Creacm.name[i] = ch;
	}
	//skip over remaining characters on the line (only first 80 characters
	//are used for the name)
	while (ch != '\n' && ch != EOF)
		ch = fgetc(infile);  // bug117 - swr - 6/29/05
	// insufficient records in input; EOF encountered
	if (ch == EOF)
	{
		creaer(25);	//generate an error 
		return;		//and quit
	}

	Creacm.name[i] = 0;	//set the last character to null (it is 
						//a null terminated string)

	fprintf(output, "\n         %s\n", Creacm.name);
}

void InputFile_class::ReadEnergySpectralDistribution()	//READ ENERGY 
//SPECTRAL DISTRIBUTION

{
	BOOLEAN eol;
	INTEGER ndred, tword;
	int i;

	static const INTEGER spect_codes[3] =
	{ CHAR2INT('s', 'p', 'e', 'c'), CHAR2INT('m', 'o', 'n', 'o'), CHAR2INT('p',
			'o', 'l', 'y') };

	tword = getwrd(TRUE, &eol, spect_codes, 1); /*getwrd returns as its value 
	 the first four letters  of the next string of alpha characters
	 in the input record  calls function 'getnxt'.
	 returns eol = TRUE when end of line is reached.*/

	if (eol)
		creaer(spect_codes[0]); //generate error if there are
	//fewer than 4 characters on the line

	Creacm.word2 = getwrd(FALSE, &eol, &(spect_codes[1]), 2); //gets the code
	//for either 'mono' or 'poly' from the input file

	if (Creacm.word2 == spect_codes[2]) //if 'poly'
	{
		// PROCESS POLYCHROMATIC DATA HERE; FOR MONOCHROMATIC GO ELESEWHERE

		Spctrm.nergy = getint(FALSE, &eol);
		if ((Spctrm.nergy <= 0) || (Spctrm.nergy > 7))
			creaer(3);

		getnxt(TRUE);

		for (i = 0; i < Spctrm.nergy; i++)
		{
			Spctrm.energy[i] = getint(FALSE, &eol);
			if (eol)
			{
				// INSUFFICIENT RECORDS IN INPUT; EOF ENCOUNTERED
				creaer(25);
				return;
			};

			Creacm.percnt[i] = getint(FALSE, &eol);
			if (eol)
			{
				// INSUFFICIENT RECORDS IN INPUT; EOF ENCOUNTERED
				creaer(25);
				return;
			}
		};

		fprintf(output,
				"\n         energy spectrum is polychromatic with %3i     energy levels",
				Spctrm.nergy);
		fprintf(output, "\n         with the following photon distribution");
		fprintf(output, "\n         energy level   percent");
		for (i = 0; i < Spctrm.nergy; i++)
		{
			fprintf(output, "\n%17i %11i", Spctrm.energy[i], Creacm.percnt[i]);
		}

		// COMPUTE THE WEIGHTS FOR EACH ENERGY LEVEL FOR AVERAGING THE
		// ABSORPTION OVER THE ENERGY LEVELS
		ndred = 0;

		for (i = 0; i < Spctrm.nergy; i++)
		{
			Spctrm.engwt[i] = Creacm.percnt[i] * (REAL) 0.01;
			ndred += Creacm.percnt[i];
			if (Creacm.percnt[i] < 0)
				ndred = -100000;
		}

		if (ndred != 100)
			creaer(4);
	}
	else	//if 'mono'
	{
		if (Creacm.word2 == spect_codes[1])
		{
			// MONOCHROMATIC DATA

			Spctrm.nergy = 1;
			Creacm.percnt[0] = 100;
			Spctrm.engwt[0] = 1.0;
			Spctrm.energy[0] = getint(FALSE, &eol);
			;
			fprintf(output,
					"\n         energy spectrum is monochromatic at energy level %5i\n",
					Spctrm.energy[0]);
		}
		else
			creaer(21);
	}
}

void InputFile_class::GetInformationABoutCreationOfPhantom(INTEGER invoke)
// GET INFORMATION ABOUT CREATION OF PHANTOM
{
	BOOLEAN eol;
	static const INTEGER crea = CHAR2INT('c', 'r', 'e', 'a');

	INTEGER tword;
	static const INTEGER phan_codes[2] =
	{ CHAR2INT('p', 'h', 'a', 'n'), CHAR2INT('a', 'v', 'e', 'r') };

	tword = getwrd(TRUE, &eol, phan_codes, 1);
	if (eol)
		creaer(phan_codes[0]);

	fprintf(output, "\n");

	Creacm.word2 = getwrd(FALSE, &eol, &(phan_codes[1]), 1);
	Creacm.picflg = (Creacm.word2 == phan_codes[1]);

	// IF PHANTOM NOT CREATED DONT READ NAVE1

	if (Creacm.picflg)
	{
		if (invoke == crea)
		{
			fprintf(output, "\n         this run will generate a phantom");
		}
		Creacm.nave1 = getint(FALSE, &eol);
		fprintf(output,
				"\n         density in each pixel is obtained as the average of %i x %i points\n",
				Creacm.nave1, Creacm.nave1);

		if ((Creacm.nave1 <= 0) || (Creacm.nave1 == Creacm.nave1 / 2 * 2))
			creaer(8);

		// READ IN GRID SIZE AND PIXEL SIZE .

		GeoPar.nelem = getint(TRUE, &eol);
		GeoPar.pixsiz = getnum(FALSE, &eol);
		fprintf(output, "\n         picture size %i x %i,  pixel size %10.4f\n",
				GeoPar.nelem, GeoPar.nelem, GeoPar.pixsiz);

		if ((GeoPar.nelem <= 0) || (GeoPar.nelem == GeoPar.nelem / 2 * 2))
			creaer(9);
		if (GeoPar.pixsiz < Consts.zero)
			creaer(10);
		GeoPar.midpix = (GeoPar.nelem + 1) / 2;
		GeoPar.area = GeoPar.nelem * GeoPar.nelem;
	}

}

// READ BACKGROUND ABSORPTION
void InputFile_class::ReadBackgroundAbsorption()
{
	BOOLEAN eol;
	int i;
	static const INTEGER hback = CHAR2INT('b', 'a', 'c', 'k');

	if (word != hback)
		creaer(hback);
	for (i = 0; i < Spctrm.nergy; i++)
	{
		Spctrm.backgr[i] = getnum(FALSE, &eol);
	}

	fprintf(output, "\n                               at levels");
	fprintf(output, "\n                               ");
	for (i = 0; i < Spctrm.nergy; i++)
	{
		fprintf(output, "%9i ", Spctrm.energy[i]);
	}

	fprintf(output, "\n          background absorption");
	for (i = 0; i < Spctrm.nergy; i++)
	{
		fprintf(output, "%9.4f ", Spctrm.backgr[i]);
	}
}

// READ RAYSUM INFORMATION
void InputFile_class::ReadRaysumInformation()
{
	BOOLEAN eol;
	//  static const INTEGER haver = CHAR2INT('a','v','e','r');  // commented out since it is unused. Lajos, Jan 13, 2005

	INTEGER tword;
	static const INTEGER rays_codes[2] =
	{ CHAR2INT('r', 'a', 'y', 's'), CHAR2INT('a', 'v', 'e', 'r') };

	tword = getwrd(TRUE, &eol, rays_codes, 1);
	if (eol)
		creaer(rays_codes[0]);

	fprintf(output, "\n");

	Creacm.word2 = getwrd(FALSE, &eol, &(rays_codes[1]), 1);
	Creacm.prjflg = (Creacm.word2 == rays_codes[1]);
}

// INPUT ENTIRE GEOMETRY DATA
void InputFile_class::ReadGeometryData(INTEGER invoke)
{
	BOOLEAN eol;
	INTEGER tword;

	static const INTEGER crea = CHAR2INT('c', 'r', 'e', 'a');

	static const INTEGER geom_codes[10] =  /// bug #135. Lajos, 08/01/2005
			{
					CHAR2INT('g', 'e', 'o', 'm'), CHAR2INT('p', 'a', 'r', 'a'),
					CHAR2INT('d', 'i', 'v', 'e'), CHAR2INT('l', 'i', 'n', 'o'), /// bug #135. Lajos, 08/01/2005
					CHAR2INT('u', 'n', 'i', 'f'), CHAR2INT('v', 'a', 'r', 'i'),
					CHAR2INT('l', 'i', 'n', 'e'), CHAR2INT('s', 't', 'r', 'i'),
					CHAR2INT('a', 'r', 'c', ' '), CHAR2INT('t', 'a', 'n', 'g') };

	fprintf(output, "\n");

	tword = getwrd(TRUE, &eol, geom_codes, 1);
	if (eol)
		creaer(geom_codes[0]);

	fprintf(output, "\n");

	// GET RAY TYPE, PARALLEL, DIVERGENT, OR LINOGRAM, UNIF, VARI, TANG OR ARC  /// bug #135. Lajos, 08/01/2005
	tword = getwrd(TRUE, &eol, &(geom_codes[1]), 3); /// bug #135. Lajos, 08/01/2005
	if (eol)
	{
		fprintf(output,
				"\n **** keyword parallel, divergent, or linogram is missing"); /// bug #135. Lajos, 08/01/2005
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	};
	Creacm.word1 = tword;

	GeoPar.lino = FALSE;  /// bug #135. Lajos, 08/02/2005

	if (tword == geom_codes[1])
	{
		tword = getwrd(FALSE, &eol, &(geom_codes[4]), 2); /// bug #135. Lajos, 08/01/2005
		if (eol)
		{
			fprintf(output, "\n **** keyword uniform or variable is missing");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
	}
	else if (tword == geom_codes[2])  /// bug #135. Lajos, 08/01/2005
	{
		tword = getwrd(FALSE, &eol, &(geom_codes[8]), 2); /// bug #135. Lajos, 08/01/2005
		if (eol)
		{
			fprintf(output, "\n **** keyword arc or tangent is missing");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
	}
	else  /// bug #135. Lajos, 08/01/2005
	{
		// assuming PARALLEL VARIABLE LINE when LINOGRAM has been specified
		GeoPar.lino = TRUE;
		Creacm.word1 = geom_codes[1];
		tword = geom_codes[5];
	}

	Creacm.word2 = tword;

	GeoPar.par = (Creacm.word1 == geom_codes[1]);
	GeoPar.div = (Creacm.word1 == geom_codes[2]); // moved out of the "if(!GeoPar.par) {...}" block. Lajos, 08/02/2005

	if (!GeoPar.par)
	{

		// DIVERGENT GEOMETRY READ SPECS

		GeoPar.line = TRUE;
		GeoPar.arc = (Creacm.word2 == geom_codes[8]); /// bug #135. Lajos, 08/01/2005
		GeoPar.tang = (Creacm.word2 == geom_codes[9]); /// bug #135. Lajos, 08/01/2005
		if (!(GeoPar.div && (GeoPar.arc || GeoPar.tang)))
			creaer(11);

		fprintf(output, "\n         rays are divergent from point sources");

		GeoPar.radius = getnum(FALSE, &eol);
		GeoPar.stod = getnum(FALSE, &eol);
		fprintf(output, "\n         source to origin distance  %10.4f",
				GeoPar.radius);

		if (GeoPar.arc)
		{
			fprintf(output,
					"\n         the detectors lie on an arc with source to detector distance = %10.4f",
					GeoPar.stod);
		}

		if (GeoPar.tang)
		{
			fprintf(output,
					"\n         the detectors lie on a st. line with source to middle detector distance = %10.4f",
					GeoPar.stod);
		}

		if (invoke != crea)
		{
			if (GeoPar.radius <= GeoPar.nelem * GeoPar.pixsiz / 1.414)
				creaer(11);
		}
	}
	else
	{
		// PARALLEL GEOMETRY  READ SPECS; IS SPACING UNIFORM OR VARIABLE

		GeoPar.uni = (Creacm.word2 == geom_codes[4]); /// bug #135. Lajos, 08/01/2005
		GeoPar.vri = (Creacm.word2 == geom_codes[5]); /// bug #135. Lajos, 08/01/2005

		if (GeoPar.lino)
			tword = geom_codes[6];  /// bug #135. Lajos, 08/01/2005
		else
		{
			tword = getwrd(FALSE, &eol, &(geom_codes[6]), 2); /// bug #135. Lajos, 08/01/2005
			if (eol)
			{
				fprintf(output, "\n **** keyword line or strip is missing");
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			}
		}
		Creacm.word3 = tword;

		GeoPar.line = (Creacm.word3 == geom_codes[6]); /// bug #135. Lajos, 08/01/2005
		GeoPar.strip = (Creacm.word3 == geom_codes[7]); /// bug #135. Lajos, 08/01/2005

		if (!(GeoPar.par && (GeoPar.strip || GeoPar.line)))
			creaer(11);
		if (!(GeoPar.uni || GeoPar.vri))
			creaer(12);

		fprintf(output, "\n         rays are parallel with");
		if (GeoPar.uni)
		{
			fprintf(output, " uniform spacing between rays");
		}
		if (GeoPar.vri)
		{
			fprintf(output, " variable spacing between rays");
		}
		if (GeoPar.strip)
		{
			fprintf(output, "\n         data collected along strips\n");
		}
		if (GeoPar.line)
		{
			fprintf(output, "\n         data collected along lines\n");
		}
	}
	fprintf(output, "\n");
}

void InputFile_class::ReadNumberOfRaysAndRaySpacing(INTEGER invoke)
{
	BOOLEAN eol;
	INTEGER melen, tword;
	REAL zisxip;

	static const INTEGER proj = CHAR2INT('p', 'r', 'o', 'j');
	static const INTEGER crea = CHAR2INT('c', 'r', 'e', 'a');

	static const INTEGER rays_codes[3] =
	{ CHAR2INT('r', 'a', 'y', 's'), CHAR2INT('u', 's', 'e', 'r'), CHAR2INT('p',
			'r', 'o', 'g') };

	if (GeoPar.lino)  /// bug #135. Lajos, 08/01/2005
	{
		// assuming RAYS USER if LINOGRAM has been specified
		if (GeoPar.nelem == 0)
		{
			fprintf(output,
					"\n **** nelem must be specified using phantom average if linogram geometry has been chosen");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
		tword = rays_codes[1];
		GeoPar.usrays = 2 * GeoPar.nelem + 1;
		GeoPar.pinc = GeoPar.pixsiz;
	}
	else
	{
		tword = getwrd(TRUE, &eol, rays_codes, 1);
		if (eol)
			creaer(rays_codes[0]);

		GeoPar.usrays = 0;

		tword = getwrd(FALSE, &eol, &(rays_codes[1]), 2);
		if (eol)
			creaer(15);

		if (tword == rays_codes[1])
		{

			GeoPar.usrays = getint(FALSE, &eol);

			if (!((GeoPar.usrays == 0) && (invoke == crea)))
			{
				if ((GeoPar.usrays % 2) <= 0)
					creaer(13);
			}
		}
		else
		{
			melen = getint(FALSE, &eol);
			zisxip = getnum(FALSE, &eol);
		}

		GeoPar.pinc = getnum(FALSE, &eol);
		if (GeoPar.pinc < Consts.zero)
			creaer(10);
	}

	if ((invoke == proj) && (GeoPar.nelem == 0))
		creaer(23);
	if (GeoPar.arc)
	{
		if ((GeoPar.pinc * GeoPar.usrays - 1.0) / GeoPar.stod > Consts.pi)
			creaer(11);
	}

	if (tword == rays_codes[2])
		GeoPar.usrays = GeoPar.numray(melen, zisxip);

	if (GeoPar.nelem > 0)
		GeoPar.snrays = GeoPar.numray(GeoPar.nelem, GeoPar.pixsiz);
	if (GeoPar.nelem == 0)
		GeoPar.snrays = GeoPar.usrays;
	if (GeoPar.usrays == 0)
		GeoPar.usrays = GeoPar.snrays;

	fprintf(output, "\n         number of rays per projection %5i",
			GeoPar.usrays);

	if (invoke == proj)
	{
		fprintf(output, "\n         snark computed number of rays %5i",
				GeoPar.snrays);
	}
	fprintf(output, "\n         at detector spacing %10.4f", GeoPar.pinc);

	// SET THE APPROPRIATE VALUES FOR THE FIRST AND LAST RAY NUMBERS
	// FOR THE USER RAY SET AND THE SNARK RAY SET
	// THE VALUE OF NRAYS IN /USER/ COMMON NOW IS NO OF USER SPECIFIED
	// RAYS IF GIVEN  OR  SNRAYS COMPUTED BY SNARK IF DEFAULT

	GeoPar.nrays = MAX0(GeoPar.snrays, GeoPar.usrays);
	GeoPar.midray = GeoPar.nrays / 2;

	GeoPar.fsnray = (GeoPar.nrays - GeoPar.snrays) / 2;
	GeoPar.lsnray = GeoPar.fsnray + GeoPar.snrays - 1;

	GeoPar.fusray = (GeoPar.nrays - GeoPar.usrays) / 2;
	GeoPar.lusray = GeoPar.fusray + GeoPar.usrays - 1;
}

// MEASUREMENT STATISTICS  --  NOISE AND SCATTER

void InputFile_class::GetMesurementStatistics()
{
	BOOLEAN eol;
	BOOLEAN perfct, noisfl;

	int n;

	static const INTEGER hquan = CHAR2INT('q', 'u', 'a', 'n');
	static const INTEGER hscat = CHAR2INT('s', 'c', 'a', 't');
	static const INTEGER hmult = CHAR2INT('m', 'u', 'l', 't');
	static const INTEGER haddi = CHAR2INT('a', 'd', 'd', 'i');

	static const INTEGER meas_codes[10] =
	{
			CHAR2INT('m', 'e', 'a', 's'), CHAR2INT('p', 'e', 'r', 'f'),
			CHAR2INT('n', 'o', 'i', 's'), CHAR2INT('s', 'e', 'e', 'd'),
			CHAR2INT('b', 'a', 'c', 'k'), CHAR2INT('q', 'u', 'a', 'n'),
			CHAR2INT('s', 'c', 'a', 't'), CHAR2INT('m', 'u', 'l', 't'),
			CHAR2INT('a', 'd', 'd', 'i'), CHAR2INT('c', 'a', 'l', 'i')
	};

	NoisePar.setQuantumNoise(0, 1.0, 1.0);  //bug221 - swr - 03/03/07

	word = getwrd(TRUE, &eol, meas_codes, 1);
	if (eol)
		creaer(meas_codes[0]);

	word = getwrd(FALSE, &eol, &(meas_codes[1]), 2);
	if (eol)
	{
		fprintf(output, "\n **** keyword perfect or noisy is missing");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	};

	perfct = (word == meas_codes[1]);
	noisfl = (word == meas_codes[2]);
	if (!noisfl)
	{

		// IF PERFECT MEASUREMENTS THEN SKIP NOISE INFORMATION
		if (!perfct)
			creaer(meas_codes[1]);

		fprintf(output, "\n         projection data are noiseless\n");

		word = getwrd(TRUE, &eol, &(meas_codes[4]), 1);
	}
	else
	{
		// NOISE INFORMATION

		fprintf(output,
				"\n         noise characteristics of projection data follow");
		fprintf(output, "\n               nature          characteristics");

		for (;;)
		{
			// CHECK TYPE OF NOISE AND READ RELEVANT DATA
			word = getwrd(TRUE, &eol, &(meas_codes[3]), 6);

			if ((word == meas_codes[3]) || (word == meas_codes[4]))
				break;

			switch (word)
			{

			case hquan:
				// QUANTUM NOISE
				NoisePar.quanmn = getnum(FALSE, &eol);
				NoisePar.quancm = getnum(FALSE, &eol);

				// GET CALIBRATION TYPE AND ECHO ON OUTPUT
				NoisePar.quanin = 1;
				if (getwrd(FALSE, &eol, &(meas_codes[9]), 1) == meas_codes[9])
					NoisePar.quanin = getint(FALSE, &eol);
				if (NoisePar.quanin == 4)
				{
					fprintf(output, "\n              Emission tomography");
				}
				else
				{

					if ((NoisePar.quanin < 1) || (NoisePar.quanin > 4))
						creaer(16);
					fprintf(output,
							"\n               quantum        mean number of photons in actual measurement %12.4e",
							NoisePar.quanmn);
					fprintf(output,
							"\n                              mean number of photons in calibration measurement is %10.4f    times above",
							NoisePar.quancm);

					if (NoisePar.quanin == 1)
					{
						fprintf(output,
								"\n                              calibration is constant for all rays in any single projection");
					}

					if (NoisePar.quanin == 2)
					{
						fprintf(output,
								"\n                              calibration is constant over all projections for any single ray");
					}

					if (NoisePar.quanin == 3)
					{
						fprintf(output,
								"\n                              calibration is variable for every ray in every projection");
					}

					if ((NoisePar.quanmn < Consts.zero)
							|| (NoisePar.quancm < Consts.zero))
						creaer(26);

					// DISTRIBUTE PHOTONS ACCORDING TO APERTURE WEIGHTS COMPUTED EARLIER

					for (n = 0; n < GeoPar.nave2; n++)
					{
						Creacm.aperwt[n] *= NoisePar.quanmn;
					}

					fprintf(output,
							"\n                              mean numbers of photons entering each substrip are");
					fprintf(output, "\n                              ");
					for (n = 0; n < GeoPar.nave2; n++)
					{
						fprintf(output, "%12.4e ", Creacm.aperwt[n]);
					}
				}
				NoisePar.setQuantumNoise();  //bug221 - swr - 03/03/07
				//}
				break;

			case hscat:
				///if(word == hscat) {
				// SCATTER NOISE
				//bug221 - swr - 03/03/07
			{
				REAL peak = getnum(FALSE, &eol);
				REAL width = getnum(FALSE, &eol);
				NoisePar.setScatter(peak, width);
				fprintf(output,
						"\n               scatter        the fraction scattered in line is %10.4f",
						peak);
				fprintf(output,
						"\n                              scatter extends to a distance of %10.4f",
						width);
				fprintf(output,
						"\n                              dying linearly from the middle");

				if ((peak < Consts.zero) || (width < Consts.zero))
					creaer(17);
				break;
			}
				///}

			case hmult:
				// MULTIPLICATIVE NOISE
				//bug221 - swr - 03/03/07
			{
				REAL mean = getnum(FALSE, &eol);
				REAL std = getnum(FALSE, &eol);
				NoisePar.setMultiplicativeNoise(mean, std);
				fprintf(output,
						"\n               multiplicative mean %10.4f     std. dev. %10.4f",
						mean, std);
				if (fabs(NoisePar.ultnmn) < Consts.zero)
					creaer(27);
				break;
			}

			case haddi:
				// ADDITIVE NOISE
				//bug221 - swr - 03/03/07
			{
				REAL mean = getnum(FALSE, &eol);
				REAL std = getnum(FALSE, &eol);
				NoisePar.setAdditiveNoise(mean, std);
				fprintf(output,
						"               additive       %10.4f     std. dev. %10.4f",
						mean, std);
				break;
			}

			default:
				creaer(18);
			}
		}
		noisfl = ((NoisePar.quanin > 0) || NoisePar.ultnfl || NoisePar.addnfl);

		if (noisfl)
		{
			if (word != meas_codes[3])
				creaer(meas_codes[3]);

			// INPUT SEED FOR RANDOM NUMBER GENERATOR

			Creacm.iseed = getint(FALSE, &eol);
			fprintf(output,
					"\n          seed for random number generator is     %10i\n",
					Creacm.iseed);

			word = getwrd(TRUE, &eol, &(meas_codes[4]), 1);
		}
	}
}

// READ NUMBER OF PROJECTIONS
// READ LIST OF PROJECTION ANGLES INTO WORK AREA

void InputFile_class::ReadProjectionAngles(BOOLEAN f11)
{
	BOOLEAN eol;
	INTEGER JJ, IANG, NUMRAYS, II, LIMIT, NN;
	REAL RAD_DEG;
	int i, j, np;
	REAL delta;
	INTEGER tword;

	BOOLEAN equisp;
	BOOLEAN uequisp;
	BOOLEAN linosp;

	static const INTEGER angl_codes[2] =  /// bug #135. Lajos, 08/01/2005
			{ CHAR2INT('a', 'n', 'g', 'l'), CHAR2INT('e', 'q', 'u', 'a') /// bug #135. Lajos, 08/01/2005
			//    CHAR2INT('l','i','n','o')  /// bug #135. Lajos, 08/01/2005
			};

	fprintf(output, "\n");

	if (GeoPar.lino)  /// bug #135. Lajos, 08/01/2005
	{
		GeoPar.prjnum = 2 * (2 * GeoPar.nelem + 1);
	}
	else
	{
		tword = getwrd(TRUE, &eol, angl_codes, 1);
		if (eol)
			creaer(angl_codes[0]);

		GeoPar.prjnum = getint(FALSE, &eol);
	}
	fprintf(output, "\n         total number of projections %5i",
			GeoPar.prjnum);

	if (GeoPar.prjnum <= 0)
		creaer(14);

	// FIRST ALLOCATE STORAGE AREA
	if (Creacm.pang != NULL)
		delete[] Creacm.pang;
	Creacm.pang = new REAL[GeoPar.prjnum];
	Anglst.Init(GeoPar.prjnum);

	// WANT TO GENERATE EQUALLY-SPACED ANGLES ONLY IF SPECIFIED

	equisp = FALSE;
	linosp = FALSE;
	uequisp = FALSE;

	if (GeoPar.lino)
		linosp = TRUE;  /// bug #135. Lajos, 08/01/2005
	else
	{
		tword = getwrd(FALSE, &eol, &(angl_codes[1]), 1); /// bug #135. Lajos, 08/01/2005
		if (tword == angl_codes[1])
		{
			equisp = TRUE;
		}
		else
		{
			uequisp = TRUE;
		}
	}

	if (equisp)
	{
		getnxt(TRUE);

		*Creacm.pang = getnum(FALSE, &eol);
		if (eol)
			creaer(25);  // added error checking. Lajos, 03/29/2005
		*(Creacm.pang + 1) = getnum(FALSE, &eol);
		if (eol)
			creaer(25);  // added error checking. Lajos, 03/29/2005
		//C     intang = intang + 1

		// CHANGED CODE FOR READING EQUALLY SPACED ANGLES TO MAKE USERS
		// LIFE BIT MORE COMFORTABLE FOR VERSION 1.02 SNARK87 10th AUG 
		// FOR flag DESCRIPTION SEE CODE AT LABEL 40 
		// -----S.SADANANDA AUG 10 1987 

		if (GeoPar.prjnum <= 1)
			creaer(20);

		delta = (Creacm.pang[1] - Creacm.pang[0]) / (GeoPar.prjnum - 1);
#ifdef FFCOMPARE
		fprintf(output,"\n %36.30f", (double) delta);
#endif
		for (np = 1; np < GeoPar.prjnum; np++)
		{
			Creacm.pang[np] = np * delta + Creacm.pang[0];
#ifdef FFCOMPARE
			fprintf(output,"\n %5i %36.30f", np+1, Creacm.pang[np]);
#endif
		}
	}

	// Modified code to enable generation of angles for linogram 
	// reconstruction
	// ----- J. Browne, May 1993

	else if (linosp)
	{
		if (!(GeoPar.par && GeoPar.vri))
		{
			fprintf(output,
					"\n **** for linogram reconstruction use PARALLEL geometry with VARIABLE detector spacing");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
		NN = (GeoPar.prjnum / 2 - 3) / 4;
		LIMIT = 2 * NN + 1;
		NUMRAYS = 4 * NN + 3;
		RAD_DEG = (REAL) 180.0 / Consts.pi;
		JJ = 0;
		for (II = 1; II <= 2; II++)
		{
			for (IANG = -LIMIT; IANG <= LIMIT; IANG++)
			{

				if (II == 1)
				{
					Creacm.pang[JJ] = RAD_DEG
							* (REAL) atan((REAL) 2.0 * IANG / NUMRAYS) + 90;
				}

				if (II == 2)
				{
					Creacm.pang[JJ] = RAD_DEG
							* (REAL) atan((REAL) 2.0 * IANG / NUMRAYS) + 180;
				}
				JJ++;

			}
		}
	}
	else if (uequisp)
	{
		if (f11)
		{ // from file11

			eol = FALSE;
			getnxt(TRUE);
			fprintf(output, "\n");
			//bug 276, jklukowska, allow free format in file11
			for (i = 0; i < GeoPar.prjnum; i++)
			{
				Creacm.pang[i] = getnum(FALSE, &eol);    // try to get number
				while (eol)
				{                    // if end of line
					getnxt(TRUE, FALSE);         // get another line
					Creacm.pang[i] = getnum(FALSE, &eol);  // try again
				}
			}

		}
		else
		{  // from input file
			eol = FALSE;
			getnxt(TRUE);

			for (i = 0; i < GeoPar.prjnum; i++)
			{
				Creacm.pang[i] = getnum(FALSE, &eol);    // try to get number
				while (eol)
				{                    // if end of line
					getnxt(TRUE);         // get another line
					Creacm.pang[i] = getnum(FALSE, &eol);  // try again
				}
			}
		}
	}

	fprintf(output, "\n         projection angles");

	for (j = 0; j < GeoPar.prjnum; j++)
	{
		if (((j % 10) == 0) && (j > 0))
			fprintf(output, "\n                          ");
		fprintf(output, " %9.4f", Creacm.pang[j]);
	}

	// CONVERT TO RADIANS AND COMPUTE SIN AND COS TABLES

	Anglst.InitDiv(Creacm.pang, GeoPar.prjnum);

	fprintf(output, "\n");
}

void InputFile_class::ReadIfoForComputionRaysums()
{
	BOOLEAN eol;
	int i, n, ns;

	GeoPar.nave2 = getint(FALSE, &eol);
	fprintf(output,
			"\n         projection data are calculated by dividing each ray interval into %i substrips",
			GeoPar.nave2);

	if ((GeoPar.nave2 <= 0) || (GeoPar.nave2 > 13)
			|| (GeoPar.nave2 == GeoPar.nave2 / 2 * 2))
		creaer(8);
	getnxt(TRUE);

	/////////////////////////////////////////////////////////////
	for (i = 0; i < GeoPar.nave2; i++)
	{ /// Is this what is needed MK ???
		GeoPar.naper[i] = getint(FALSE, &eol);
		if (eol == TRUE)
		{
			// INSUFFICIENT RECORDS IN INPUT; EOF ENCOUNTERED
			creaer(25);
		}
	}

	fprintf(output, "\n         with aperture (substrip) weights");
	for (n = 0; n < GeoPar.nave2; n++)
	{
		fprintf(output, " %5i", GeoPar.naper[n]);
	}

	// GET SUM OF THE APERTURE WEIGHTS FOR LATER USE
	ns = 0;
	for (n = 0; n < GeoPar.nave2; n++)
	{
		if (GeoPar.naper[n] < 0)
			creaer(19);
		ns += GeoPar.naper[n];
	}
	if (ns <= 0)
		creaer(19);

	// COMPUTE NORMALIZED APERTURE WEIGHTS FOR USE IN CREATING
	// PROJECTION DATA
	for (n = 0; n < GeoPar.nave2; n++)
	{
		Creacm.aperwt[n] = ((REAL) GeoPar.naper[n]) / ((REAL) (ns));
	}
}

// READ OBJECTS MAKING UP THE PHANTOM

void InputFile_class::ReadObjects(INTEGER invoke)
{
	int otype;
	int i, j, n;

	REAL densit;

	INTEGER itype, tword;
	REAL cxt;
	REAL cyt;
	REAL ut;
	REAL vt;
	REAL angt;

	REAL scale;
	REAL phi;
	BOOLEAN eol;
	REAL dent[7];

	SNARK_Object_Node *f_object, *t_object;

	static const INTEGER crea = CHAR2INT('c', 'r', 'e', 'a');

	static const INTEGER object_codes[8] =
	{
			CHAR2INT('o', 'b', 'j', 'e'), CHAR2INT('e', 'l', 'i', 'p'),
			CHAR2INT('r', 'e', 'c', 't'), CHAR2INT('t', 'r', 'i', 'a'),
			CHAR2INT('s', 'e', 'g', 'm'), CHAR2INT('s', 'e', 'c', 't'),
			CHAR2INT('l', 'a', 's', 't'), CHAR2INT('d', 'e', 'n', 's') };

	tword = getwrd(TRUE, &eol, object_codes, 1);
	if (tword != object_codes[0])
		creaer(object_codes[0]);

	fprintf(output, "\n         description of objects");
	fprintf(output, "\n                                                                density at levels");
	fprintf(output, "\n numb type  x-coord  y-coord x-length y-length    angle  av dens");
	for (i = 0; i < Spctrm.nergy; i++)
		fprintf(output, " %9i", Spctrm.energy[i]);

	// READ IN OBJECT PARAMETERS IN TEMPORARY LOCATIONS SO AS NOT TO
	// CLOBBER WORK AREA WHILE DOING INPUT FOR RDPICT AND RDPROJ

	if (invoke == crea)
	{
		f_object = new SNARK_Object_Node;
		t_object = f_object;
	};

	for (i = 0;; i++)
	{
		getnxt(TRUE);           // read line of input
		itype = getwrd(FALSE, &eol, &(object_codes[1]), 6);

		if (eol)
		{
			fprintf(output, "\n **** keyword elip, rect, tria, segm, sect or last is missing");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		// CHECK TO SEE IF  LAST  OBJECT HAS BEEN READ

		if (itype == object_codes[6])
			break;

		cxt = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter cx of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		cyt = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter cy of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		ut = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter u of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		vt = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter v of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		angt = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter ang of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		dent[0] = getnum(FALSE, &eol);
		if (eol)
		{
			fprintf(output, "\n **** parameter den(0) of object %s is missing", int2str(itype));
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};

		// IF POLYCHROMATIC READ IN ABSORPTION FOR OTHER LEVELS

		if (Spctrm.nergy > 1)
		{
			getnxt(TRUE);           // read line of input
			getwrd(FALSE, &eol, &(object_codes[7]), 1);

			if (eol)
			{
				fprintf(output, "\n **** keyword dens is missing");
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			for (j = 1; j < Spctrm.nergy; j++)
			{
				dent[j] = getnum(FALSE, &eol);
				if (eol)
				{
					fprintf(output, "\n **** parameter den(%d) of object %s is missing", j, int2str(itype));
					fprintf(output, "\n **** program aborted\n");
					exit(-1);
				}
			}
		}

		// ALSO COMPUTE THE WEIGHTED AVERAGE ABSORPTION

		densit = 0.0;
		for (j = 0; j < Spctrm.nergy; j++)
			densit += Spctrm.engwt[j] * dent[j];

		fprintf(output, "\n %4i %s %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f", i + 1, int2str(itype), cxt, cyt, ut, vt, angt, densit);
		for (j = 0; j < Spctrm.nergy; j++)
			fprintf(output, " %9.4f", dent[j]);

		// IDENTIFY OBJECT TYPE

		for (otype = SOT_elip; otype <= SOT_sect; otype++)
		{
			if (itype == object_codes[otype + 1])
				break;
		}

		// IF INVOKED BY RDPICT OR RDPROJ NO NEED TO REMEMBER THE ABOVE VALUE
		if (invoke == crea)
		{
			if ((ut < Consts.zero) || (vt < Consts.zero))
				creaer(6);

			// TRANSFER VALUES TO A TEMPORARY LIST

			t_object->data.type = (SNARK_Object_type) otype;
			for (j = 0; j < Spctrm.nergy; j++)
				t_object->data.den1[j] = dent[j];
			t_object->data.denav = densit;
			t_object->data.cx = cxt;
			t_object->data.cy = cyt;
			t_object->data.u = ut;
			t_object->data.v = vt;
			t_object->data.ang = angt;
			phi = (REAL) (angt * Consts.pi / 180.0);
			t_object->data.sinang = (REAL) sin(phi);
			t_object->data.cosang = (REAL) cos(phi);

			t_object->next = new SNARK_Object_Node;
			t_object = t_object->next;
		}
	};

	if (invoke == crea)
	{
		MAX_NUMBER_OF_OBJECTS = i;
		objects = new SNARK_Object[MAX_NUMBER_OF_OBJECTS];

		t_object = f_object;
		for (i = 0; i < MAX_NUMBER_OF_OBJECTS; i++)
		{
			objects[i].type = t_object->data.type;
			for (j = 0; j < Spctrm.nergy; j++)
				objects[i].den1[j] = t_object->data.den1[j];
			objects[i].denav = t_object->data.denav;
			objects[i].cx = t_object->data.cx;
			objects[i].cy = t_object->data.cy;
			objects[i].u = t_object->data.u;
			objects[i].v = t_object->data.v;
			objects[i].ang = t_object->data.ang;
			objects[i].sinang = t_object->data.sinang;
			objects[i].cosang = t_object->data.cosang;

			t_object = t_object->next;
			delete f_object;
			f_object = t_object;
		};
	};

	// GRAB THE SCALE FACTOR FROM THE LAST CARD

	scale = getnum(FALSE, &eol);
	if (eol)
	{
		fprintf(output, "\n **** scale factor is missing");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	};

	fprintf(output,
			"\n         scale factor multiplying object densities %10.4f",
			scale);
	mean = scale;

	if (fabs(scale) < Consts.zero)
		creaer(7);

	// IF   PICT   OR   PROJ   NO NEED TO SCALE

	if (invoke == crea)
	{
		// GET THE NUMBER OF OBJECTS READ IN

		Creacm.nobj = i;
		if (Creacm.nobj != 0)
		{
			// SCALE DENSITIES FIRST
			for (n = 0; n < Creacm.nobj; n++)
			{
				objects[n].denav *= scale;
				for (j = 0; j < Spctrm.nergy; j++)
					objects[n].den1[j] *= scale;
			}
		}
	}
	fprintf(output, "\n");

	//jk 11/18/2007 introducing variability
	//GRAB THE VARIABILITY VALUE FROM THE LAST CARD
	//jklukowska 12/9/8, bug 274, varseed should be read as an integer, not real number
	Creacm.varseed = getint(FALSE, &eol);
	if (eol)
	{
		Creacm.varseed = 0;
	}

	fprintf(output, "\n         seed set to %d", Creacm.varseed);


	variability = getnum(FALSE, &eol);
	if (eol)
	{
		variability = 0;
	}
	else if (variability < 0)
	{
		fprintf(output, "\n **** inhomogeneity should be >= 0");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	}

	if (variability != 0)
		variab_set = TRUE;
	else if (variability == 0)
		variab_set = FALSE;

	fprintf(output, "\n         inhomogeneity set to %10.4f", variability);

}

// added by Lajos, Jan 25, 2005
/*
 ECHOLINE PRINTS THE CONTENTS OF THE CURRENT INPUT LINE (I.E. THE ARRAY 'DATA') TO THE OUTPUT.
 IF flag == true, THE OUTPUT WILL START WITH '<*>'. OTHERWISE, '<#>' WILL BE USED.
 */
void InputFile_class::echoline(BOOLEAN flag)
{
	if (flag)
		fprintf(output, "\n     <*> %s", data);
	else
		fprintf(output, "\n     <#> %s", data);
}
