/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/qfilt.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 COMPUTE THE VALUE OF THE FILTER FUNCTION FOR THE RHO-FILTERED
 LAYERGRAM ALGORITHM
 */

/*      
 THE IMPLEMENTED FILTERS ARE-
 FILTER=0     BRACEWEL  ABS(R)*BOX(R/(2.0*CUTOFF))
 FILTER=1     SHEPP     ABS(R)*SINC(R/(2.0*CUTOFF))
 *BOX(R/(2.0*CUTOFF))
 FILTER=2     COSINE    ABS(R)*COS(PI*R/(2.0*CUTOFF))
 *BOX(R/(2.0*CUTOFF))
 FILTER=3     HAMMING   ABS(R)*(ALPHA+(1.0-ALPHA)*COS(PI*R))
 *BOX(R/2.0)
 WHERE ALPHA = CUTOFF
 */

#include <cstdlib>
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "qfilt.h"
#include "consts.h"
#include "uiod.h"

REAL qfilt(REAL r, REAL cutoff, INTEGER filter)
{
	if (!((filter < 0) || (filter > 3)))
	{
		switch (filter)
		{

		case 0:
			if (fabs(r) > cutoff)
				return 0.0;
			return (REAL) fabs(r);

		case 1:
			if (fabs(r) > cutoff)
				return 0.0;
			return (REAL) (cutoff * sin(Consts.pid2 * fabs(r) / cutoff)
					/ Consts.pid2);

		case 2:
			if (fabs(r) > cutoff)
				return 0.0;
			return (REAL) (fabs(r) * cos(Consts.pid2 * r / cutoff));

		case 3:
			if (fabs(r) > 1.0)
				return 0.0;
			return (REAL) (fabs(r)
					* (cutoff + (1.0 - cutoff) * cos(Consts.pi * r)));
		}
	}

	// INVALID FILTER, OUTPUT ERROR MESSAGE AND STOP

	fprintf(output, "\n **** filter = %d is not a valid filter type for qfilt routine", filter);
	fprintf(output, "\n **** program aborted\n");
	exit(500);
}
