/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_deltad.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

                 R#*D*D*R
      DELTA = ---------------------
              G#*D*(D*M#*A*M*D)*D*G

    NOTE: R = M#*A*(M*X-P)

    AOPT=1            AOPT=2           AOPT=3             AOPT=4

    (R=G=V)           (R=G)            (R=V)

1.                                     PRTN=TWONRM        SAME
2.  TWONRM=D**2*R**2  SAME             SAME               SAME
3.                                     GMA=TWONRM/PRTN    SAME
4.                                     G=R+GMA*G          SAME
5.  CG=D**2*G         SAME             SAME               SAME
6.  V=M#*A*M*CG       SAME             SAME               SAME
7.  GTQG=CG*V         SAME             SAME               SAME
          TWONRM
8.  DELTA=------      SAME             SAME               SAME
           GTQG

                      NOTE:            NOTE:              NOTE:
                      TWONRM AND       FOR CONSTANT
                      DELTA ARE NOT    DELTA ONLY         SAME
                      COMPUTED FOR     V IS COMPUTED
                      CONSTANT DELTA
*/

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include "blob.h"

#include "quad.h"

void quad_class::deltad(
REAL* r,
REAL* g,
REAL* v,
REAL* cg,
REAL* d,
REAL* delta,  // corr. hstau
		REAL* gma,     //corr. hstau
		INTEGER* list,
		REAL* weight,
		BOOLEAN *alg)
{
	static REAL twonrm;     // set static. hstau
	static REAL prtn;     // set static. hstau
	REAL gtqg;
	INTEGER i;

	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	if (iterx == 1)
		twonrm = 0.0;  //corr? hstau

//C1.

	if (!((bopt <= 2) && (popt <= 0)))
	{
		prtn = twonrm;

//C2.

		twonrm = 0.0;
		for (i = 0; i < area; i++)
		{
			if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
			{
				twonrm += d[i] * r[i] * r[i];
			}
		}

		if (popt >= 1)
		{
			fprintf(output, "\n\n      squared norm of the residual is %15.6e",
					twonrm);     //wei, bug 191, 12/2005
		}

		if (twonrm <= Consts.zero)
		{
			fprintf(output,
					"\n\n      norm of the residual is zero, iteration stops"); //wei, bug 191, 12/2005
			*alg = TRUE;
			return;
		}
	}

	if (aopt > 2)
	{
		if (iterx == 1)
		{

//C3.
			for (i = 0; i < area; i++)
			{
				g[i] = r[i];
			}
			goto L4;
			//corr.hstau
		}
		if (bopt == 3)
			*gma = twonrm / prtn; //gma = twonrm/prtn;

//C4.

		for (i = 0; i < area; i++)
		{
			g[i] = r[i] + (*gma) * g[i];  // corr. hstau
;
		}

//C5.
	}
	L4:  // corr.hstau
	for (i = 0; i < area; i++)
	{
		cg[i] = d[i] * g[i];


	}

	// RETURN FOR AOPT .LE. 2 AND BOPT .LE. 2

	if ((bopt <= 2) && (aopt <= 2))
		return; // corr.hstau
//C6.

	mtamx(cg, v, list, weight);

	if (bopt <= 2)
		return;

//C7.

	gtqg = 0.0;
	for (i = 0; i < area; i++)
	{
		if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
		{
			gtqg += cg[i] * v[i];
		}
	}


	// CHECK FOR ZERO DENOMINATOR

	if (fabs(gtqg) > Consts.zero)
	{

//C8.
		*delta = twonrm / gtqg;

		return;
	}

	fprintf(output, "\n***zero denominator when computing delta iteratively, program stops***"); //not selfexplained hstau
	*alg = TRUE;
	return;
}
