/*
***********************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_eval_recon4.c $
$LastChangedRevision: 137 $
$Date: 2014-07-15 16:38:33 -0400 (Tue, 15 Jul 2014) $
$Author: bprommegger $
***********************************************************
*/

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "experimenter.h"
#include "errorc.h"
#include "read_eval_recon4.h"

/*
 this function reads from the eval file the kullback-leibler distance

 INPUTS:
 iteration - iteration number
 keywrd - keyword which defines the algorithm whose reconstruction is to be read in.

 OUTPUT:
 recon - double precision variable which stores the normalized root mean
 square distance between the (clipped) reconstruction and the
 (clipped) phantom.
*/

void read_eval_recon4(int iteration, char* keywrd, double* klds, double* wsqd)
{
	int area = 0;
	int global = 0;
	int iter = 0;
	double average = 0;
	double distance = 0;
	double relerr = 0;
	double variance = 0;
	double stddev = 0;
	double residual = 0;
	double kld = 0;
	double local_wsqd = 0;
	char string[MAXLINESIZE];
	char test[MAXLINESIZE];
	FILE *evalfile;

	if ((evalfile = fopen("eval", "r")) == NULL) errorc("in read_eval_recon4.c: error in opening file ", "eval");

	while (1)
	{
		/* Read eval file until a line which contains the string
		 'experimenter evaluation 2' is reached. If end of file is
		 reached then keyword was not found so terminate program. */

		fgets(string, sizeof(string), evalfile);

		if (strstr(string, "experimenter evaluation 2") != NULL)
		{
			break;
		}

		if (feof(evalfile))
		{
			fflush(stdout);
			sprintf(string,"Error in read_eval_recon4.c: keyword %s not found in file ","experimenter evaluation 2");
			errorc(string, "eval");
		}
	}

	while (1)
	{
		/* Read eval file until a line which contains the string
		 'execution name' is reached. If this line contains the
		 keyword, break out of the while loop. If end of file is
		 reached then keyword was not found so terminate program. */

		fgets(string, sizeof(string), evalfile);

		if (strstr(string, "execution name") != NULL)
		{
			sscanf(string, "%*s%*s %s", test);
			if (strncasecmp(test, keywrd, 4) == 0)
			{
				break;
			}
		}

		if (feof(evalfile))
		{
			fflush(stdout);
			sprintf(string,"Error in read_eval_recon4.c: keyword %s not found in file ",keywrd);
			errorc(string, "eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line of numbers is
		 reached, then break out of while loop. If end of file
		 is encountered then terminate program. */
		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf", &iter, &area, &average, &distance, &relerr, &variance, &stddev, &residual, &kld, &local_wsqd) == 10)
		{
			break;
		}
		if (feof(evalfile))
		{
			errorc("Error in read_eval_recon4.c: end of file reached in file eval. Hint: Have the iteration flags been set to 3?","");
		}
	}

	/* If necessary, search the remaining lines of numbers until
	 the appropriate iteration number is found. Then save the
	 distance and break out of while loop. If approriate
	 iteration number is not found then terminate program. */
	if (iteration != 0)
	{
		/* if this first line of numbers has correct iteration number, return*/
		if (iter == iteration)
		{
			*klds = kld;
			*wsqd = local_wsqd;
			fclose(evalfile);
			return;
		}

		while (1)
		{
			fgets(string, sizeof(string), evalfile);
			sscanf(string, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf", &iter, &area, &average, &distance, &relerr, &variance, &stddev, &residual, &kld, &local_wsqd);
			if (iter == iteration)
			{
				*klds = kld;
				*wsqd = local_wsqd;
				break;
			}
			if (feof(evalfile))
			{
				sprintf(string,"Error in read_eval_recon4.c: %s --> iteration %d not found in file", keywrd, iteration);
				errorc(string, " eval");
			}
		}
	}

	// iteration == 0: the last iteration should be processed instead of a literal interpretation of 0
	else
	{
		char prev[MAXLINESIZE];

		while (1)
		{
			strcpy(prev, string);
			fgets(string, sizeof(string), evalfile);
			if (strlen(string) < 3)
			{
				break;
			}
			if (feof(evalfile))
			{
				sprintf(string, "Error in read_eval_recon4.c: %s --> iteration %d not found in file",keywrd, iteration);
				errorc(string, " eval");
			}
		}

		sscanf(prev, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf", &iter, &area, &average, &distance, &relerr, &variance, &stddev, &residual, &kld, &local_wsqd);
		*klds = kld;
		*wsqd = local_wsqd;
	}

	fclose(evalfile);
}
