/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                              S N A R K   1 4                              *
 *                                                                           *
 *                     A PICTURE RECONSTRUCTION PROGRAM                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

 stopex.cpp,v 1.2 2008/08/25 16:11:09 jklukowska Exp

DETERMINE THE ALGORITHM STOPPING CRITERION.  SET TERM TO REFLECT
IT.  FOR AN ITERATION COUNT, TERM IS THE NEGATIVE OF THAT NUMBER.
FOR A TEST NAME, TERM CONTAINS THE INDEX.

*/


#include <cstdio>
#include <cstdlib>
#include "blkdta.h"
#include "consts.h"
#include "term.h"
#include "trm1.h"
#include "trm2.h"
#include "trm3.h"
#include "trm4.h"
#include "trm5.h"
#include "trm6.h"
#include "trm_weighted_squared_distance.h"
#include "uiod.h"
#include "int2str.h"
#include "infile.h"
#include "stopex.h"

trm1_class trm1;
trm2_class trm2;
trm3_class trm3;
trm4_class trm4;
trm5_class trm5;
trm6_class trm6;
trm_weighted_squared_distance trm_weighted_squared_distance_inst;

termtest_class* TrmClasses[] =
{
  &trm1,
  &trm2,
  &trm3,
  &trm4,
  &trm5,
  &trm6,
  &trm_weighted_squared_distance_inst
};

void stopex()
{
  static const INTEGER code[] =
  {
    CHAR2INT('i','t','e','r'),
    CHAR2INT('t','e','r','m')
  };

  static const INTEGER testn[] =
  {
    CHAR2INT('t','r','m','1'),
    CHAR2INT('t','r','m','2'),
    CHAR2INT('v','a','r','i'),
    CHAR2INT('m','l','s','t'),
    CHAR2INT('k','l','d','s'),
    CHAR2INT('r','e','s','i'),
    CHAR2INT('w','s','q','d')
  }; 
  INTEGER num_termination_criteria = 7;

  INTEGER i;

  INTEGER word;
  BOOLEAN eol;

  // GET THE TERMINATION CRITERION
  word = InFile.getwrd(FALSE, &eol, code, 2);

  switch(word)
  {
  case CHAR2INT('i','t','e','r'):

    // GET THE NUMBER OF ITERATIONS
    word = InFile.getint(FALSE, &eol);

    if (word <= 0)
    {
      fprintf(output, "\n **** number of iterations must be greater than zero");
      fprintf(output, "\n **** program aborted\n");
      exit(-1);
    }

    Term.iterations = word;
    Term.termInstance = NULL;
    fprintf(output, "\n         %4i iterations", word);
    return;

  case CHAR2INT('t','e','r','m'):

    // READ THE TEST NAME
    word = InFile.getwrd(FALSE, &eol, testn, num_termination_criteria);

    for(i = 0; i < num_termination_criteria; i++) {
      if(word == testn[i]) {
	fprintf(output, "\n         termination test %s", int2str(word));

	Term.termInstance = TrmClasses[i];
	Term.termInstance->Init();
	Term.iterations = 0;
        return;

      }
    }

    fprintf(output, "\n **** unknown test %s", int2str(word));
    fprintf(output, "\n **** program aborted\n");
    exit(-1);

    return;

  default:
    fprintf(output, "\n **** unknown command %s, ITER or TERM must be specified", int2str(word));
    fprintf(output, "\n **** program aborted\n");
    exit(-1);
    return;
  }
}      
