/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                               *
 *                         S N A R K   1 4                       *
 *                                                               *
 *                A PICTURE RECONSTRUCTION PROGRAM               *
 *                                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

  stru_acc.c,v 1.4 2009/06/01 03:33:26 jklukowska Exp

 Previously part of SuperSNARK
 */

#include "malloc.h"
#include "experimenter.h"
#include "stru_acc.h"
#include "read_eval_recon1.h"
#include "read_eval_phantom1.h"

/* ----------------------------- stru_acc.c -------------------------------

        This function computes the structural accuracy.

INPUTS:
  itr1 - array of length niters containing the iteration numbers for the
         first algorithm.
  itr2 - array of length niters containing the iteration numbers for the
         second algorithm.
  niters - number of iterations to be compared
  keywrd1 - string containing the keyword which defines the first algorithm.
  keywrd2 - string containing the keyword which defines the second algorithm.

OUTPUTS:
  str_acc1 - array of length niters containing the structural accuracy for 
             the first algorithm.
  str_acc2 - array of length niters containing the structural accuracy for
             the second algorithm.

 */

void stru_acc(int* itr1, int* itr2, int niters, char* keywrd1, char* keywrd2, double* str_acc1, double* str_acc2)
//int *itr1,*itr2,niters;
//char *keywrd1,*keywrd2;
//double *str_acc1,*str_acc2;
{
    double *phantom, *recon1, *recon2;
    int *regions, numstr, i, j, *strarea;

    /* read in abnormality index for phantom structures */

    read_eval_phantom1(&regions, &numstr, &phantom, &strarea);

    /* allocate memory to store abnormality indexes for
       structures in the reconstructions*/

    recon1 = (double *) malloc(numstr * sizeof (double));
    recon2 = (double *) malloc(numstr * sizeof (double));

    for (i = 0; i < niters; i++) { /* read in abnormality indexes for structures in
		   the reconstructions. Initialize the arrays
		   which will store the structural accuracy */

        read_eval_recon1(&itr1[i], keywrd1, numstr, recon1);
        read_eval_recon1(&itr2[i], keywrd2, numstr, recon2);
        str_acc1[i] = 0.0;
        str_acc2[i] = 0.0;
        for (j = 0; j < numstr; j++) { /* compute structural accuracy for both algorithms */

            str_acc1[i] = str_acc1[i] + fabs(recon1[j] - phantom[j]);
            str_acc2[i] = str_acc2[i] + fabs(recon2[j] - phantom[j]);
        }
        str_acc1[i] = -str_acc1[i] / numstr; /* structural */
        str_acc2[i] = -str_acc2[i] / numstr; /* accuracy */
    }

    free(phantom); /* free memory for next function call */
    free(recon1);
    free(recon2);
    free(strarea);

}
