/** @file image.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "variables.hpp"
#include "image.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <qmessagebox.h>

Snarkimage::Snarkimage()
{
  image=NULL;
  max=-100000;
  min=100000;
  isloaded=false;
  fileformat=DIGDataFormat_ASCII;
}

Snarkimage::Snarkimage(int imagenumber,DIGDataFormat format)
{
  int i,j;
  double *tmpimg;
  max=-100000;
  min=100000;
  tmpimg=(double *)malloc(sizeof(double)*Sizey*Sizex);
  image=matrix(0,Sizey,0,Sizex);
  if(!format) { /* format=0 (ASCII)  */
    fileformat=DIGDataFormat_ASCII;
    digrecfil.SelectArraySet(imageindexes[imagenumber][0]);
    digrecfil.SelectArray(imageindexes[imagenumber][1]);
    digrecfil.GetArrayData(tmpimg);
    for(i=0;i<Sizey;i++) {
// bug #141 fix
      for(j=0;j<Sizex;j++) {
	image[Sizey-i-1][j]=tmpimg[i*Sizex+j];
	// bug! image[i][j]=tmpimg[i*Sizey+j]; fixed 2004 apr
        if(image[Sizey-i-1][j]>max)
	  max=image[Sizey-i-1][j];
        if(image[Sizey-i-1][j]<min)
	  min=image[Sizey-i-1][j];
      }
    }
  }
  else { /* format=1 (BINARY) */
    fileformat=DIGDataFormat_BINARY;
    digrecfil.SelectArraySet(imageindexes[imagenumber][0]);
    digrecfil.SelectArray(imageindexes[imagenumber][1]);
    digrecfil.GetArrayData(tmpimg);
    //    for(i=0;i<Sizey;i++) {
    for(i=Sizey-1; i>=0; i--) { // bug #141 fix
      for(j=0;j<Sizex;j++) {
	//bug! image[i][j]=tmpimg[i*Sizey+j]; fixed 2004 apr
	image[Sizey-i-1][j]=tmpimg[i*Sizex+j];
        if(image[Sizey-i-1][j]>max)
	  max=image[Sizey-i-1][j];
        if(image[Sizey-i-1][j]<min)
	  min=image[Sizey-i-1][j];
      }
    }
  }
 isloaded=true;
 if(max>globalmax) 
   globalmax=max;
 if(min<globalmin) 
   globalmin=min;
 free(tmpimg);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Snarkimage::~Snarkimage()
{
    // no need to delete child widgets, Qt does it all for us
}

void Snarkimage::readImage(int imagenumber,DIGDataFormat format)
{
  int i,j;
  double *tmpimg;

  max=-100000;
  min=100000;
  tmpimg=(double *)malloc(sizeof(double)*Sizey*Sizex);
  image=matrix(0,Sizey,0,Sizex);
  if(!format) { /* format=0 (DIGDataFormat_ASCII)  */
    digrecfil.SelectArraySet(imageindexes[imagenumber][0]);
    digrecfil.SelectArray(imageindexes[imagenumber][1]);
    digrecfil.GetArrayData(tmpimg);
    for(i=0;i<Sizey;i++) {
      // bug #141 fix
      for(j=0;j<Sizex;j++) {
	image[Sizey-i-1][j]=tmpimg[i*Sizey+j];
        if(image[Sizey-i-1][j]>max)
	  max=image[Sizey-i-1][j];
        if(image[Sizey-i-1][j]<min)
	  min=image[Sizey-i-1][j];
      }
    }
  }
  else { /* format=1 (DIGDataFormat_BINARY) */
    digrecfil.SelectArraySet(imageindexes[imagenumber][0]);
    digrecfil.SelectArray(imageindexes[imagenumber][1]);
    digrecfil.GetArrayData(tmpimg);
    for(i=0;i<Sizey;i++) {
      // bug #141 fix
      for(j=0;j<Sizex;j++) {
	image[Sizey-i-1][j]=tmpimg[i*Sizey+j];
        if(image[Sizey-i-1][j]>max)
	  max=image[Sizey-i-1][j];
        if(image[Sizey-i-1][j]<min)
	  min=image[Sizey-i-1][j];
      }
    }
  }
 isloaded=true;
 if(max>globalmax) 
   globalmax=max;
 if(min<globalmin) 
   globalmin=min;
 free(tmpimg);
}

double** Snarkimage::getImage()
{
  return image;
}

double Snarkimage::getMax()
{
  return max;
}

double Snarkimage::getMin()
{
  return min;
}

bool Snarkimage::isLoaded()
{
  return isloaded;
}

double** Snarkimage::matrix(int nrl,int nrh,int ncl,int nch)
/*  Allocates a double pt array with range [npl .. nph][nrl .. nrh][ncl .. nch]  */
{
   int j;
   double **m;

   m = (double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
   if (!m) {
       printf("Allocation failure in matrix() (y)\n");
       exit(-1);
   }
   m -= nrl;
   for(j=nrl; j<=nrh; j++) {
     m[j] = (double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
     if(!m[j]) {
       printf("Allocation failure in matrix() (x)\n");
       exit(-1);
     }
     m[j] -= ncl;
   }
   return m;
}

