/** @file linesimages.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
    
    modification: Joanna Klukowska 1/13/2009
	    added choice of grayScale display for plot
	    in addition to color display
*/

#include "linesimages.hpp"
#include "selectimages.hpp"
#include "variables.hpp"

#include <stdio.h>
#include <malloc.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

extern Snarkimage **imageset;

/*  "Lines" (plotting of pixel values in a row or column) dialog for multiple images
 *  Constructs a linesimageswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'fl'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
linesimageswindow::linesimageswindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{    if ( !name )
	setName( "lineswindow" );
    resize( 340, 550 ); 
    setMinimumSize( QSize( 340, 550 ) );
    setMaximumSize( QSize( 340, 550 ) );
    setCaption( tr( "Multiple Images Profile" ) );
    selected=false;
    minval=globalmin;
    maxval=globalmax;
    rowcolbox = new QListBox( this, "rowcolbox" );
    rowcolbox->insertItem( tr( "rows" ) ); // rows first. => 0 => false  [ x-axis == columns ] [sic]
    rowcolbox->insertItem( tr( "columns" ) );   // columns later => 1 => true [ x-axis == rows ] [sic]
    rowcolbox->setGeometry( QRect( 20, 10, 80, 50 ) );
    rowcolbox->setSelectionMode(QListBox::Single); 
    rowcolbox->setSelected(0,true);
    QObject::connect(rowcolbox,SIGNAL(highlighted(int)),this,SLOT(updateXAxis(int)));
    imagesbutton = new QPushButton( this, "imagesbutton" );
    imagesbutton->setGeometry( QRect( 130, 20, 140, 40 ) ); 
    imagesbutton->setText( tr( "Select Images" ) );
    QObject::connect(imagesbutton,SIGNAL(clicked()),this,SLOT(callSelectImages()));

    line1 = new QLineEdit( this, "line1" );
    line1->setGeometry( QRect( 20, 90, 60, 30 ) ); 
    line2 = new QLineEdit( this, "line2" );
    line2->setGeometry( QRect( 100, 90, 60, 30 ) ); 
    line3 = new QLineEdit( this, "line3" );
    line3->setGeometry( QRect( 180, 90, 60, 30 ) ); 
    line4 = new QLineEdit( this, "line4" );
    line4->setGeometry( QRect( 260, 90, 60, 30 ) ); 

    label1 = new QLabel( this, "label1" );
    label1->setGeometry( QRect( 33, 70, 35, 20 ) ); 
    label1->setText( tr( "row1" ) );
    label2 = new QLabel( this, "label2" );
    label2->setGeometry( QRect( 113, 70, 35, 20 ) ); 
    label2->setText( tr( "row2" ) );
    label3 = new QLabel( this, "label3" );
    label3->setGeometry( QRect( 193, 70, 35, 20 ) ); 
    label3->setText( tr( "row3" ) );
    label4 = new QLabel( this, "label4" );
    label4->setGeometry( QRect( 273, 70, 35, 20 ) ); 
    label4->setText( tr( "row4" ) );

    title =  new QLineEdit( this, "title" );
    title->setGeometry( QRect( 20, 150, 300, 30 ) );
    QString nameprojQStr = nameproj;
    title->setText(nameprojQStr.simplifyWhiteSpace());
    titlelabel = new QLabel( this, "titlelabel" );
    titlelabel->setGeometry( QRect( 160, 130, 60, 20 ) ); 
    titlelabel->setText( tr( "Title" ) );

//      subtitle =  new QComboBox( this, "subtitle" );
//      subtitle->setGeometry( QRect( 20, 210, 140, 30 ) );
//      subtitle->insertItem("Column/Line #",-1);
//      subtitle->insertItem("Empty",-1);
//      subtitlelabel = new QLabel( this, "subtitlelabel" );
//      subtitlelabel->setGeometry( QRect( 60, 190, 100, 20 ) ); 
//      subtitlelabel->setText( tr( "Subtitle" ) );

//      gridstyle = new QComboBox(this,"gridstyle");
//      gridstyle->setGeometry( QRect( 180, 210, 140, 30 ) ); 
//      gridstyle->insertItem("Solid",-1);
//      gridstyle->insertItem("None",-1);
//      gridstyle->insertItem("Dash",-1);
//      gridstyle->insertItem("Dot",-1);

//      gridstylelabel = new QLabel( this, "gridstylelabel" );
//      gridstylelabel->setGeometry( QRect( 210, 190, 210, 20 ) ); 
//      gridstylelabel->setText( tr( "Grid Style" ) );

    xaxis =  new QLineEdit( this, "xaxis" );
    xaxis->setEnabled(false); // user doesn't change it
    xaxis->setGeometry( QRect( 20, 240, 140, 30 ) );
    xaxis->setText("Columns"); // default setting

    yaxis =  new QLineEdit( this, "yaxis" );
    yaxis->setEnabled(false);
    yaxis->setGeometry( QRect( 180, 240, 140, 30 ) );
    yaxis->setText("Pixel Values");

    xaxislabel = new QLabel( this, "xaxislabel" );
    xaxislabel->setGeometry( QRect( 70, 220, 60, 20 ) );
    xaxislabel->setText( tr( "X Axis" ) );

    yaxislabel = new QLabel( this, "yaxislabel" );
    yaxislabel->setGeometry( QRect( 230, 220, 60, 20 ) );
    yaxislabel->setText( tr( "Y Axis" ) );

    minx = new QLineEdit( this, "minx" );
    minx->setGeometry( QRect( 20, 300, 140, 30 ) );
    minx->setText("0");

    QString s;
    s.setNum(Sizex-1);
    maxx = new QLineEdit( this, "maxx" );
    maxx->setGeometry( QRect( 180, 300, 140, 30 ) );
    maxx->setText(s);

    s.setNum(globalmin);
    miny = new QLineEdit( this, "miny" );
    miny->setGeometry( QRect( 20, 360, 140, 30 ) );
    miny->setText(s);

    s.setNum(globalmax);
    maxy = new QLineEdit( this, "maxy" );
    maxy->setGeometry( QRect( 180, 360, 140, 30 ) );
    maxy->setText(s);

    minxlabel = new QLabel( this, "minxlabel" );
    minxlabel->setGeometry( QRect( 60, 280, 60, 20 ) );
    minxlabel->setText( tr( "Min Col" ) ); // OK, so shoot me for hardcoding

    maxxlabel = new QLabel( this, "maxxlabel" );
    maxxlabel->setGeometry( QRect( 220, 280, 60, 20 ) );
    maxxlabel->setText( tr( "Max Col" ) );

    minylabel = new QLabel( this, "minylabel" );
    minylabel->setGeometry( QRect( 70, 340, 60, 20 ) );
    minylabel->setText( tr( "Min Y" ) );

    maxylabel = new QLabel( this, "maxylabel" );
    maxylabel->setGeometry( QRect( 230, 340, 60, 20 ) );
    maxylabel->setText( tr( "Max Y" ) );


    // jk 1/13/2009 adding grayScale option for graphs
    // jklukowska 6/7/2009 modified to be radio button with option of either
    // color or grayscale
    colorbuttongroup = new QButtonGroup(this, "colorbuttongroup");
    colorbuttongroup->setTitle(tr("Draw graphs in "));
    colorbuttongroup->setGeometry(QRect(0, 400, 340, 75));
    colorbuttongroup->setAlignment(int( QButtonGroup::AlignHCenter));
    grayScaleButton = new QRadioButton(colorbuttongroup, "grayScaleButton");
    grayScaleButton->setGeometry(QRect(30, 25, 130, 20));
    grayScaleButton->setText(tr("  grayscale"));
    grayScaleButton->setEnabled(true);
    grayScaleButton->setChecked(false);
    colorButton = new QRadioButton(colorbuttongroup, "colorButton");
    colorButton->setGeometry(QRect(190, 25, 130, 20));
    colorButton->setText(tr(" color"));
    colorButton->setEnabled(true);
    colorButton->setChecked(true);

    //Line3 = new QFrame( this, "Line3" );
    //Line3->setGeometry( QRect( 0, 485, 340, 16 ) ); 
    //Line3->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    showbutton = new QPushButton( this, "showbutton" );
    showbutton->setGeometry( QRect( 20, 490, 100, 40 ) ); 
    showbutton->setText( tr( "Show" ) );
    QObject::connect(showbutton,SIGNAL(clicked()),this,SLOT(checkAndAccept()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 220, 490, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject())); 
} // --linesimageswindow ctor

/*  
 *  Destroys the object and frees any allocated resources
 */
linesimageswindow::~linesimageswindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void linesimageswindow::updateXAxis(int i) // i is 1 for row, 0 for cols
{
  if(i==0) { // so x axis is the other.
    label1->setText("row1");
    label2->setText("row2");
    label3->setText("row3");
    label4->setText("row4");
    xaxis->setText("Columns");
    minxlabel->setText("Min Col");
    maxxlabel->setText("Max Col");
  } else {
    label1->setText("col1");
    label2->setText("col2");
    label3->setText("col3");
    label4->setText("col4");
    xaxis->setText("Rows");
    minxlabel->setText("Min Row");
    maxxlabel->setText("Max Row");
  }
} // --linesimageswindow::updateXAxis()

/** Starts a window for selecting the images to be shownfor a lines command
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void linesimageswindow::callSelectImages() 
{
  int i,*siarray;
  QString s;

  selectimageswindow* nw=new selectimageswindow(this,"Select Images for Line Displaying",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    siarray=nw->getSelectedImages();
    if(siarray[0]==0 && !phantombutton->isChecked()) 
      QMessageBox::information(this,"SnarkDisplay","Warning!\n"
			       "No image was selected.\n");
    else {
      selected=true;
      selectedimages = (int*) malloc((unsigned) (siarray[0]+1)*sizeof(int));
      if(!selectedimages) {
        QMessageBox::information(this,"SnarkDisplay","Error!\n"
                              "Allocation failure in getSelectedImages()\n");
      }
      else {
        selectedimages[0]=siarray[0];
        for(i=1;i<=siarray[0];i++) {
          selectedimages[i]=siarray[i];
	  if(!imageset[siarray[i]]->isLoaded())
            imageset[siarray[i]]->readImage(siarray[i],dataformat);
	}
      }
    }
    /*   s.sprintf("%f",globalmin);
    miny->setText(s);
    s.sprintf("%f",globalmax);
    maxy->setText(s);  */
  }
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int* linesimageswindow::getSelectedImages()
{
  return selectedimages;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getLinetype()
{
  return rowcolbox->currentItem();
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getLine1()
{
  int l;
  bool ok;
  l=line1->text().toInt(&ok,10);
  if(ok) 
    return l;
  else
    return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getLine2()
{
  int l;
  bool ok;
  l=line2->text().toInt(&ok,10);
  if(ok) 
    return l;
  else
    return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getLine3()
{
  int l;
  bool ok;
  l=line3->text().toInt(&ok,10);
  if(ok) 
    return l;
  else
    return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getLine4()
{
  int l;
  bool ok;
  l=line4->text().toInt(&ok,10);
  if(ok) 
    return l;
  else
    return NaI;
}


/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
bool linesimageswindow::isPhantomChecked()
{
  if(phantombutton->isChecked())
    return true;
  else
    return false;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
QString linesimageswindow::getTitle()
{
  return title->text();
}

//  /** 
//  @param void
//  @author Bruno M. Carvalho
//  @version 1.0 */ 
//  int linesimageswindow::getSubtitle()
//  {
//    return subtitle->currentItem();
//  }

//  /** 
//  @param void
//  @author Bruno M. Carvalho
//  @version 1.0 */ 
//  int linesimageswindow::getGridstyle()
//  {
//    return gridstyle->currentItem();
//  }

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
QString linesimageswindow::getXAxis()
{
  return xaxis->text();
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
QString linesimageswindow::getYAxis()
{
  return yaxis->text();
}


/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getMinX()
{
  bool ok;
  int t;
  t=minx->text().toInt(&ok,10);
  if(ok)
    return t;
  else 
    return 0;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int linesimageswindow::getMaxX()
{
  bool ok;
  int t;
  t=maxx->text().toInt(&ok,10);
  if(ok)
    return t;
  else 
    return Sizex-1;
}


/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
double linesimageswindow::getMinY()
{
  bool ok;
  double t;
  t=miny->text().toDouble(&ok);
  if(ok)
    return t;
  else 
    return minval;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
double linesimageswindow::getMaxY()
{
  bool ok;
  double t;
  t=maxy->text().toDouble(&ok);
  if(ok)
    return t;
  else 
    return maxval;
}

void linesimageswindow::checkAndAccept()
{
  if(selected) {
    accept();
  }
  else {
    QMessageBox::information(this,"SnarkDisplay","Error!\n"
			     "No image was selected.\n");
  }
}

/**
@param void
@author Joanna Klukowska (added 1/13/2009)
@version 1.0 */
bool linesimageswindow::isGrayScaleChecked() {
    return (grayScaleButton->isChecked());
}
