/** @file openimages.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "variables.hpp"
#include "openimages.hpp"
#include "displaywindow.hpp"
#include "SnarkDisplay.hpp"

#include <stdio.h>
#include <qapplication.h>
#include <qframe.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qwidget.h>

// extern displaywindow **displaywindowset;

/* 
 *  Constructs a openimageswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
openimageswindow::openimageswindow( displaywindow** displaywindowset_in, QWidget* parent,  const char* name, bool modal, WFlags fl , int x , int y )
  //    : QDialog( parent, name, modal, fl )
  : QWidget( parent, name, fl )
{
    QPoint topleft;
    displaywindowset = displaywindowset_in;
    if ( !name )
	setName( "openimageswindow" );
    resize( 540, 350 );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 540, 350 ) );
    setMaximumSize( QSize( 540, 350 ) );
    setCaption( tr( "Open Images" ) );

    Parent=parent;

    topleft.setX(x);
    topleft.setY(y);
    this->move(topleft);
    openbutton = new QPushButton( this, "openbutton" );
    openbutton->setGeometry( QRect( 10, 15, 140, 40 ) ); 
    openbutton->setText( tr( "Open" ) );

    QObject::connect(openbutton,SIGNAL(clicked()) ,this, SLOT(openImages()));

    selectbutton = new QPushButton( this, "selectbutton" );
    selectbutton->setGeometry( QRect( 10, 65, 140, 40 ) ); 
    selectbutton->setText( tr( "Select All" ) );
    QObject::connect(selectbutton,SIGNAL(clicked()),this,SLOT(selectAll()));
    //    QObject::connect(openbutton,SIGNAL(clicked()) ,parent, SLOT(signalAllDifference(bool)));


    clearselectbutton = new QPushButton( this, "clearselectbutton" );
    clearselectbutton->setGeometry( QRect( 10, 115, 140, 40 ) ); 
    clearselectbutton->setText( tr( "Clear Selected" ) );
    QObject::connect(clearselectbutton,SIGNAL(clicked()),this,SLOT(clearSelected()));

    imageslistbox = new QListBox( this, "imageslistbox" );
    imageslistbox->setSelectionMode(QListBox::Multi);
    QFont f = QFont("courier",10,25,false);
    f.setFixedPitch(true);
    imageslistbox->setFont(f);
    for(int i=0;i<numimages;i++)
      imageslistbox->insertItem(imagetitles[i]);
    imageslistbox->setGeometry( QRect( 160, 15, 370, 250 ) ); 

    Line6 = new QFrame( this, "Line6" );
    Line6->setGeometry( QRect( 0, 280, 540, 16 ) ); 
    Line6->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    closebutton = new QPushButton( this, "closebutton" );
    closebutton->setGeometry( QRect( 380, 300, 100, 40 ) ); 
    closebutton->setText( tr( "Close" ) );
    //    QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(reject()));
    QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(close()));
}


/*  
 *  Destroys the object and frees any allocated resources
 */
openimageswindow::~openimageswindow()
{
    // no need to delete child widgets, Qt does it all for us
}


void openimageswindow::openImages()
{
  //  std::cerr << "++openimageswindow::openImages()" << std::endl;
  int i,x,y;
  QPoint topleft;
  topleft=this->pos();
  x=topleft.x();
  y=topleft.y();
  for(i=0;i<numimages;i++) {
//     std::cerr << "in openimages() for loop, i=" << i << std::endl;
//     std::cerr << "displaywindowset[i]=" << (displaywindowset[i]) 
// 	      << " active: " << (displaywindowactive[i] ? "yes" : "no")
// 	      << std::endl;
    // the new way--should only happen in the first pass of the same recfil
    //    if(!displaywindowset[i]) {
      //      displaywindowset[i]=new displaywindow(0,"",false,0,dataformat,i,x,y);
    //    }
    if((imageslistbox->isSelected(i)) && (!displaywindowactive[i])) {
      //      std::cerr << "SHOWING for i=" << i << std::endl;
      // fixed HUGE bug while trying to fix bug #119 (which was a while ago)
      //   displaywindow* nw=new displaywindow(desk,"",false,0,dataformat,i,x,y);
      //   nw->show();
      x+=30;
      y+=30;
      if(y>(maxSizey-630)) {
	y=30;
      }
      if(x>(maxSizex-50)) {
	x = 30;
      }
      if(!displaywindowset[i]) {
	displaywindowset[i]=new displaywindow(0,"",false,0,dataformat,i,x,y);
      }
      displaywindowactive[i]=true;
      displaywindowset[i]->show();
      displaywindowset[i]->updateZoom(); // trick to get right size scroll pane
    }
  }
  //  std::cerr << "--openimageswindow::openImages()" << std::endl;
} // --openimageswindow::openImages()


void openimageswindow::setImagesNames()
{
  int i;
  
  imageslistbox->clear();
  for(i=0;i<numimages;i++) 
    imageslistbox->insertItem(imagetitles[i]);
}


void openimageswindow::selectAll()
{
  imageslistbox->selectAll(true);
}


void openimageswindow::clearSelected()
{
  imageslistbox->selectAll(false);
}
