/** @file SnarkInput.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    modifications made June 13, 2008 by Joanna Klukowska
	addition of variability to snark14 (two extra fields in LAST line
	changed occurances of snark05 to snark14
	removed SAVE and MODIFY options from RUN since snark14 does not have file12
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "SnarkInput.h"
#include "icons.h"
#include "variables.h"

#include "basis.h"
#include "create.h"
#include "display.h"
#include "evaluate.h"
#include "execute.h"
#include "experiment.h"
#include "lines.h"
#include "mode.h"
#include "picture.h"
#include "projection.h"
#include "punch.h"
#include "select.h"
#include "skunk.h"
#include "stop.h"
#include "super.h"
#include "trace.h"

#include <qstring.h>
#include <qlabel.h>
#include <qrect.h>
#include <qintdict.h>
#include <qlistbox.h>
#include <qtoolbar.h>
#include <qstatusbar.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>

char *myTag = "si140703";

int edittype=1;

/** 
 *  Constructs a SnarkInput which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
SnarkInput::SnarkInput(QWidget* parent, const char* name, WFlags fl)
    : QMainWindow(parent, name, fl)
{ 
  printf("snark14Input version %s\n", myTag);

  entries=0;
  resize(482,665);
  setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
  setMinimumSize(QSize(482,665));

  inputfilename=QString("./snark_input");

  QPixmap comments_icon( ( const char** ) comments_icon_data );
  QPixmap trace_icon( ( const char** ) trace_icon_data );
  QPixmap mode_icon( ( const char** ) mode_icon_data );
  QPixmap create_icon( ( const char** ) create_icon_data );
  QPixmap picture_icon( ( const char** ) picture_icon_data );
  QPixmap projection_icon( ( const char** ) projection_icon_data );
  QPixmap select_icon( ( const char** ) select_icon_data );
  QPixmap basis_icon( ( const char** ) basis_icon_data );
  QPixmap stop_icon( ( const char** ) stop_icon_data );
  QPixmap super_icon( ( const char** ) super_icon_data );
  QPixmap execute_icon( ( const char** ) execute_icon_data );
  QPixmap evaluate_icon( ( const char** ) evaluate_icon_data );
  QPixmap display_icon( ( const char** ) display_icon_data );
  QPixmap punch_icon( ( const char** ) punch_icon_data );
  QPixmap lines_icon( ( const char** ) lines_icon_data );
  QPixmap skunk_icon( ( const char** ) skunk_icon_data );
  QPixmap end_icon( ( const char** ) end_icon_data );
  QPixmap run_icon( ( const char** ) run_icon_data );
  QPixmap open_icon( ( const char** ) open_icon_data );
  QPixmap save_icon( ( const char** ) save_icon_data );
  QPixmap saveas_icon( ( const char** ) saveas_icon_data );
  QPixmap newfile_icon( ( const char** ) newfile_icon_data );

  QMenuBar* menubar=menuBar();

  filemenu=new QPopupMenu(menubar,"File Menu");
  QString t="&Open";
  filemenu->insertItem(t,this,SLOT(openFile()));
  t="&New";
  filemenu->insertItem(t,this,SLOT(newInputFile()));
  t="&Save";
  filemenu->insertItem(t,this,SLOT(saveFile()));
  t="S&ave as";
  filemenu->insertItem(t,this,SLOT(saveFileAs()));
  t="&Quit";
  filemenu->insertItem(t,qApp,SLOT(quit()));  

  commandsmenu=new QPopupMenu(menubar,"Commands Menu");
  t="&* Comments Window";
  commandsmenu->insertItem(t,this,SLOT(callComment()));
  t="&Trace Window";
  commandsmenu->insertItem(t,this,SLOT(callTrace()));
  t="&Mode Window";
  commandsmenu->insertItem(t,this,SLOT(callMode()));
  t="&Create Window";
  commandsmenu->insertItem(t,this,SLOT(callCreate()));
  t="&Picture Window";
  commandsmenu->insertItem(t,this,SLOT(callPicture()));
  t="p&Rojection Window";
  commandsmenu->insertItem(t,this,SLOT(callProjection()));
  t="&Select Window";
  commandsmenu->insertItem(t,this,SLOT(callSelect()));
  t="&Basis Function Window";
  commandsmenu->insertItem(t,this,SLOT(callBasis()));
  t="st&Op Window";
  commandsmenu->insertItem(t,this,SLOT(callStop()));
  t="s&Uperiorize Window";
  commandsmenu->insertItem(t,this,SLOT(callSuper()));
  t="&Execute Window";
  commandsmenu->insertItem(t,this,SLOT(callExecute()));
  t="e&Valuate Window";
  commandsmenu->insertItem(t,this,SLOT(callEvaluate()));
  t="&Display Window";
  commandsmenu->insertItem(t,this,SLOT(callDisplay()));
  t="p&Unch Window";
  commandsmenu->insertItem(t,this,SLOT(callPunch()));
  t="&Lines Window";
  commandsmenu->insertItem(t,this,SLOT(callLines()));
  t="s&Kunk Function Window";
  commandsmenu->insertItem(t,this,SLOT(callSkunk()));
  t="E&nd Command";
  commandsmenu->insertItem(t,this,SLOT(addEnd()));

  runsnarkmenu=new QPopupMenu(menubar,"Run Snark Menu");
  t="&Run SNARK";
  runsnarkmenu->insertItem(t,this,SLOT(runSnark()));

  helpmenu=new QPopupMenu(menubar,"Help Menu");
  t="&About";
  helpmenu->insertItem(t,this,SLOT(about()));

  menubar->insertItem("&File",filemenu);
  menubar->insertItem("&Commands",commandsmenu);
  menubar->insertItem("&SNARK",runsnarkmenu);
  menubar->insertSeparator();
  menubar->insertItem("&Help",helpmenu);

  /** Add graphical buttons to the interface */  
  QToolBar *tb1 = new QToolBar("label",this,Top,false,"fileiconsbar");
  tb1->setHorizontalStretchable(true);

  QToolButton* ii=new QToolButton(open_icon,"Open File",QString::null,this,SLOT(openFile()),tb1,"Open File");

  ii=new QToolButton(newfile_icon,"New File",QString::null,this,SLOT(newInputFile()),tb1,"New File");

  ii=new QToolButton(save_icon,"Save File",QString::null,this,SLOT(saveFile()),tb1,"Save File");

  ii=new QToolButton(saveas_icon,"Save File as",QString::null,this,SLOT(saveFileAs()),tb1,"Save File as");

  QToolBar *tb2 = new QToolBar("label",this,Left,false,"iconsbar");
  tb2->setVerticalStretchable(true);

  QToolBar *tb3 = new QToolBar("label",this,Left,false,"iconsbar2");
  tb3->setVerticalStretchable(true);

  ii=new QToolButton(comments_icon,"Comments",QString::null,this,SLOT(callComment()),tb2,"Experiment Comments");

  ii=new QToolButton(trace_icon,"Trace Window",QString::null,this,SLOT(callTrace()),tb2,"Trace Window");

  ii=new QToolButton(mode_icon,"Mode window",QString::null,this,SLOT(callMode()),tb2,"Mode window");

  ii=new QToolButton(create_icon,"Create Window",QString::null,this,SLOT(callCreate()),tb2,"Create Window");

  ii=new QToolButton(picture_icon,"Picture window",QString::null,this,SLOT(callPicture()),tb2,"Picture window");

  ii=new QToolButton(projection_icon,"Projection Window",QString::null,this,SLOT(callProjection()),tb2,"Projection Window");

  ii=new QToolButton(select_icon,"Select Window",QString::null,this,SLOT(callSelect()),tb2,"Select Window");

  ii=new QToolButton(basis_icon,"Basis window",QString::null,this,SLOT(callBasis()),tb2,"Basis window");

  ii=new QToolButton(stop_icon,"Stop Window",QString::null,this,SLOT(callStop()),tb2,"Stop Window");

  ii=new QToolButton(super_icon,"Superiorize Window",QString::null,this,SLOT(callSuper()),tb2,"Superiorize Window");

  ii=new QToolButton(execute_icon,"Execute Window",QString::null,this,SLOT(callExecute()),tb3,"Execute Window");

  ii=new QToolButton(evaluate_icon,"Evaluate Window",QString::null,this,SLOT(callEvaluate()),tb3,"Evaluate Window");

  ii=new QToolButton(display_icon,"Display window",QString::null,this,SLOT(callDisplay()),tb3,"Display window");  

  ii=new QToolButton(punch_icon,"Punch window",QString::null,this,SLOT(callPunch()),tb3,"Punch window");

  ii=new QToolButton(lines_icon,"Lines window",QString::null,this,SLOT(callLines()),tb3,"Lines window");

  ii=new QToolButton(skunk_icon,"Skunk Window",QString::null,this,SLOT(callSkunk()),tb3,"Skunk Window");

  ii=new QToolButton(end_icon,"End Command",QString::null,this,SLOT(addEnd()),tb3,"End Command");

  ii=new QToolButton(run_icon,"Run SNARK14",QString::null,this,SLOT(runSnark()),tb3,"Run SNARK14");

  currentinput = new QMultiLineEdit(this,"current input");
  setCentralWidget(currentinput);

  QLabel* ibl = new QLabel( statusBar(), "tracelabel" );
  ibl->setGeometry( QRect( 10, 0, 130, 30 ) ); 
  ibl->setText( tr( "Insert command:" ) );

  QButtonGroup* insertbuttongroup = new QButtonGroup( statusBar(), "tracebuttongroup" );
  insertbuttongroup->setGeometry( QRect( 150, 0, 300, 60 ) );
  insertbuttongroup->setFrameStyle(QFrame::NoFrame);
  insertbuttongroup->setAlignment( int( QButtonGroup::AlignVCenter ) );
  QObject::connect(insertbuttongroup,SIGNAL(clicked(int)),this,SLOT(setInsertType(int)));

  QRadioButton* int1button = new QRadioButton(insertbuttongroup,"int1button");
  int1button->setGeometry( QRect( 0, 0, 100, 30 ) ); 
  int1button->setText( tr( "At the end" ) );
  //  int1button->setChecked(true);

  QRadioButton* int2button = new QRadioButton(insertbuttongroup,"int2button");
  int2button->setGeometry( QRect( 110, 0, 130, 30 ) ); 
  int2button->setText( tr( "After cursor line" ) );
  int2button->setChecked(true);

  sleep(1);

  //  delete splash;
}

/** Calls file dialog and opens file
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::setInsertType(int e)
{
  edittype=e;
}


/** Calls file dialog and opens file
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::openFile()
{
  QString tempbuffer,tempinputfilename;
  char tempstring[500];
  int i,l;

  tempinputfilename=QFileDialog::getOpenFileName(QString::null,"Snark input files (*.in)\n All files (*)");
  if(tempinputfilename) {
    inputfilename=tempinputfilename;
    if((inputfile=fopen(inputfilename,"r+"))==NULL) {
      printf("Error opening %s\n",(const char *)inputfilename);
    } 
    else { /* fseek(inputfile,0L,SEEK_END); */
      tempbuffer=QString();
      l=currentinput->numLines();
      for(i=0;i<l;i++)
        currentinput->removeLine(0);
      while(fgets(tempstring,500,inputfile)) {
	tempbuffer.append(QString(tempstring));
      }
      currentinput->insertLine(tempbuffer,-1);
      fclose(inputfile);
    }
  }
}

/** Erases current content for a new input file
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::newInputFile()
{
  while(currentinput->numLines()>1)
    currentinput->removeLine(0);
  currentinput->removeLine(0);
  inputfilename=QString("./snark_input");
}

/**  Saves file 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::saveFile()
{
  int i,l;
  QString t;

  if((inputfile=fopen(inputfilename,"w"))==NULL) {
  QMessageBox::information(this,"SnarkInput","Error!\n"
                                 "Could not open file %s\n",(const char *)inputfilename);
  } 
  else {
    l=currentinput->numLines();
    for(i=0;i<l;i++) {
      t=currentinput->textLine(i);
      fprintf(inputfile,"%s\n",t.latin1());
    }
    fclose(inputfile);  
  }
}

/**  Calls file dialog and saves file as
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::saveFileAs()
{
  int i,l;
  QString t;

  outputfilename=QFileDialog::getSaveFileName(QString::null,"Snark input files (*.in)\n All files (*)");
  if(outputfilename) {
    inputfilename=outputfilename;
    if((inputfile=fopen(inputfilename,"w"))==NULL) {
    QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Could not open file %s\n",(const char *)inputfilename);
    } 
    else {
      l=currentinput->numLines();
      for(i=0;i<l;i++) {
        t=currentinput->textLine(i);
        fprintf(inputfile,"%s\n",t.latin1());
      }
      fclose(inputfile);
    }
  }
}

/** Calls SNARK with current input file
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::runSnark()
{ 
  QString todo;
  printf("SNARK14 is run on file %s\n",(const char *)inputfilename);
  char tmpfilename[100];
  sprintf(tmpfilename, "/tmp/snark_input_%d_%d", getuid(), geteuid());
  char command[150];
  if((inputfile=fopen(tmpfilename, "w"))==NULL) {
    printf("Error opening for read-write %s\n",(const char *)tmpfilename);
  }
  todo=currentinput->text();
  fprintf(inputfile,"%s\n",todo.latin1());
  fflush(inputfile);
  fclose(inputfile);  
  // Added by deniz to fix bug 85
  // these tests return 0 for pass, so we use some deMorgan :)
  if(system("test -x ./bin/snark14UserDefined") ||  // file exists and executable
     system("test -s ./bin/snark14UserDefined") ||  // exists and has non-zero size
     system("test -f ./bin/snark14UserDefined")) {  // exists and is regular (not a directory)
// std::cout << "file is not a non-zero-size executable non-directory" << std::endl;
    // run snark14 as usual
    sprintf(command, "snark14 < %s", tmpfilename);
    //    system("snark14 < /tmp/snark_input ");
    system(command);
  } else {
// std::cout << "file is a non-zero-size executable non-directory" << std::endl;
    // run ./bin/snark14UserAlg
    sprintf(command, "./bin/snark14UserDefined < %s", tmpfilename);
    //    system("./bin/snark14UserAlg < /tmp/snark_input ");
    system(command);
  }
  // don't clean-up... potentially dangerous... let files be.
  //  sprintf(command, "rm -f %s", tmpfilename);
  //  system(command);
}

/** Calls SnarkInput manual
@param void
@author Bruno M. Carvalho
@version 1.0 
COMMENTED OUT BY DENIZ SARIOZ in line with "new requirements"
*/
// void SnarkInput::help()
// {
//   char command[80];
//   // char *snarkinputbase;
//   sprintf(command,"acroread %s/doc/SnarkInput.pdf &",SNARKINPUTBASE);
//   /* snarkinputbase=getenv("HOME");
//   printf("SNARKINPUTBASE=%s\n",snarkinputbase);
//   snarkinputbase=getenv("SNARKBASE");
//   printf("SNARKINPUTBASE=%s\n",snarkinputbase);
//   sprintf(command,"acroread $SNARKINPUTBASE/doc/SnarkInput.pdf &"); */
//   system(command);
// }

/** Calls Snark manual
@param void
@author Bruno M. Carvalho
@version 1.0 
COMMENTED OUT BY DENIZ SARIOZ in line with "new requirements"
*/
// void SnarkInput::SNARKManual()
// {
//   char command[80];
//   sprintf(command,"acroread %s/doc/Snark_manual.pdf &",SNARKBASE);
//   system(command); 
// }

/** Calls about window
@param void
@author Bruno M. Carvalho & Deniz Sarioz
@version 2.0 */
void SnarkInput::about()
{
  QMessageBox::information(this,"snark14Input",
			      "snark05Input was developed by\n"
			   "Bruno M. Carvalho and Deniz Sarioz\n"
			   "2003--2005\n"
			   "converted to snark14Input by Joanna Klukowska\n"
			   "June 2008\n"
			   "converted to snark14Input by Bernhard Prommegger\n"
			   "July 2014\n"
                           "copyright (QPL) by CUNY-GC-CS-DIG\n"
			   "http://www.dig.cs.gc.cuny.edu");
}

/** Calls comment window
@param void
@author Bruno M. Carvalho
@version 1.0 */
void SnarkInput::callComment()
{
  QString line;
  int l,col;

  experimentwindow* nw=new experimentwindow(this,"Experiment's Comments Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a trace window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callTrace()
{ 
  QString line;
  int l,col;

  tracewindow* nw=new tracewindow(this,"Trace Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a mode window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callMode()
{
  QString line;
  int l,col;

  modewindow* nw=new modewindow(this,"Mode Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a create window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callCreate() 
{  
  QString line1,line2,line3,line4,line5,line6,line7,line8;
  QStringList divline;
  int ell,col;

  createwindow* nw=new createwindow(this,"Create Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line1=nw->getOutput1();
    line2=nw->getOutput2();
    line3=nw->getOutput3();
    line4=nw->getOutput4();
    line5=nw->getOutput5();
    if(nw->isRaysumAveragePresent()) {
      line6=nw->getOutput6();
      line7=nw->getOutput7();
      line8=nw->getOutput8();
    }
    if(edittype) {
      currentinput->getCursorPosition(&ell,&col);
      if((currentinput->numLines()) == (ell+1)) { // if cursor on last line of text
	currentinput->insertLine("",ell+1); // hack so that stuff below will work
      }
      if(nw->isRaysumAveragePresent()) {
	currentinput->insertLine(line8,ell+1);
	currentinput->insertLine(line7,ell+1);
	currentinput->insertLine(line6,ell+1);
      }
      currentinput->insertLine(line5,ell+1);
      currentinput->insertLine(line4,ell+1);
      currentinput->insertLine(line3,ell+1);
      currentinput->insertLine(line2,ell+1);
      currentinput->insertLine(line1,ell+1);
    }
    else {
      currentinput->insertLine(line1);
      currentinput->insertLine(line2);
      currentinput->insertLine(line3);
      currentinput->insertLine(line4);
      currentinput->insertLine(line5);
      if(nw->isRaysumAveragePresent()) {
	currentinput->insertLine(line6);
	currentinput->insertLine(line7);
	currentinput->insertLine(line8);
      }
    }
  }
}

/** Starts a picture window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callPicture()
{
  QString line;
  int l,col;

  picturewindow* nw=new picturewindow(this,"Picture Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a projection window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callProjection() 
{
  QString line;
  int l,col;

  projectionwindow* nw=new projectionwindow(this,"Projection Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a select window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callSelect() 
{
  QString line;
  int l,col;

  selectwindow* nw=new selectwindow(this,"Select Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a basis window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callBasis()
{  
  QString line;
  int l,col;

  basiswindow* nw=new basiswindow(this,"Basis Function Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  } 
}

/** Starts a stop window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callStop() 
{
  QString line;
  int l,col;

  stopwindow* nw=new stopwindow(this,"Stop Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {    
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

// Starts a superiorize window
void SnarkInput::callSuper()
{
  QString line;
  int l,col;

  superwindow* nw=new superwindow(this,"Superiorize Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a execute window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callExecute()
{
  QString line;
  int l,col;

  executewindow* nw=new executewindow(this,"Execute Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a evaluate window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callEvaluate() 
{
  QString line;
  int l,col;

  evaluatewindow* nw=new evaluatewindow(this,"Evaluate Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a display window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callDisplay() 
{
  QString line;
  int l,col;

  displaywindow* nw=new displaywindow(this,"Display Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a punch window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callPunch()
{  
  QString line;
  int l,col;

  punchwindow* nw=new punchwindow(this,"Punch Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a lines window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callLines() 
{
  QString line;
  int l,col;

  lineswindow* nw=new lineswindow(this,"Lines Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Starts a skunk window
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::callSkunk() 
{
  QString line;
  int l,col;

  skunkwindow* nw=new skunkwindow(this,"Skunk Window",true);
  int c=nw->exec();
  if(c==QDialog::Accepted) {
    line=nw->getOutput();
    if(edittype) {
      currentinput->getCursorPosition(&l,&col);
      currentinput->insertLine(line,l+1);
    }
    else
      currentinput->insertLine(line);
  }
}

/** Adds an end command
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void SnarkInput::addEnd()
{  
  int l,col;

  if(edittype) {
    currentinput->getCursorPosition(&l,&col);
    currentinput->insertLine(QString("END"),l+1);
  }
  else
    currentinput->insertLine(QString("END"));
}

/** Starts the application
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
int main(int argc,char* argv[])
{
  QApplication snarkapp(argc,argv);
 
  SnarkInput* mw=new SnarkInput();
  snarkapp.setMainWidget(mw);
  mw->show();
  return snarkapp.exec();
}
