/** @file projection.h
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#ifndef PROJECTIONWINDOW_H
#define PROJECTIONWINDOW_H

#include <qvariant.h>
#include <qdialog.h>
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout;  
class QCheckBox; 
class QComboBox;
class QButtonGroup;
class QFrame;
class QLabel;
class QLineEdit;
class QMultiLineEdit;
class QPushButton;
class QRadioButton;
class QTabWidget;

class projectionwindow : public QDialog
{ 
    Q_OBJECT

public:
    projectionwindow( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~projectionwindow();
    QString getOutput();

    QTabWidget* tabwidget1;

    QWidget* projTypeWidget;
    QLabel* projTypeLabel;

    QLabel* projectionlabel;
    QButtonGroup* projectionbuttongroup;
    QRadioButton* oldbutton;
    QRadioButton* realbutton;
    QCheckBox* bhcbutton;
    QMultiLineEdit * bhcwindow;
    QRadioButton* pseudobutton;

    QTabWidget* pseudoCommandsWidget;

    QWidget* idheaderWidget;
    QLineEdit* idheader;
    QLabel* idheaderlabel;

    QWidget* geometryWidget;
    QLabel* geometrylabel;
    QButtonGroup* geometrybuttongroup;
    QRadioButton* parallelbutton;
    QRadioButton* divergentbutton;
    QButtonGroup* parallelbuttongroup;
    QRadioButton* uniformbutton;
    QRadioButton* variablebutton;
    QButtonGroup* parallelbuttongroup2;
    QRadioButton* stripbutton;
    QRadioButton* linebutton;
    QButtonGroup* divergentbuttongroup;
    QRadioButton* arcbutton;
    QRadioButton* tangentbutton;
    QLineEdit* sourcetoorigin;
    QLabel* sourcetooriginlabel;
    QLineEdit* sourcetodetector;
    QLabel* sourcetodetectorlabel;
    QLabel* raysumslabel;
    QButtonGroup* raysumsbuttongroup;
    QRadioButton* userbutton;
    QLineEdit* userrays;
    QLabel* userrayslabel;
    QRadioButton* programbutton;
    QLineEdit* raysumsnelem;
    QLabel* raysumsnelemlabel;
    QLineEdit* raysumspixelsize;
    QLabel* raysumspixelsizelabel;
    QLineEdit* detectorspacing;
    QLabel* detectorspacinglabel;
    QLabel* angleslabel;
    QButtonGroup* anglesbuttongroup;
    QLineEdit* prjnum;
    QLabel* prjnumlabel;
    QRadioButton* linobutton;
    QRadioButton* equalspacingbutton;
    QLineEdit* firstangle;
    QLabel* firstanglelabel;
    QLineEdit* lastangle;
    QLabel* lastanglelabel;
    QRadioButton* nonebutton;
    QLineEdit* angles;  // DS: make it a regular line edit
    QWidget* measurementWidget;

    // set 7
    QWidget* measurementRun;
    QLabel* measurementlabel;
    QButtonGroup* measurementbuttongroup;
    QRadioButton* perfectbutton;
    QRadioButton* noisybutton;
    QFrame* noisyframe;
    QCheckBox* quantumbutton;
    QLineEdit* quanmn;
    QLabel* quanmnlabel;
    QLineEdit* quancm;
    QLabel* quancmlabel;
    QLabel* calibrationlabel;
    QComboBox* quanin;
    QLabel* quaninlabel;
    QCheckBox* scatterbutton;
    QLineEdit* sctnpk;
    QLabel* sctnpklabel;
    QLineEdit* sctnwd;
    QLabel* sctnwdlabel;
    QCheckBox* additivebutton;
    QLineEdit* addnmn;
    QLabel* addnmnlabel;
    QLineEdit* addnsd;
    QLabel* addnsdlabel;
    QCheckBox* multiplicativebutton;
    QLineEdit* multnmn;
    QLabel* multnmnlabel;
    QLineEdit* multnsd;
    QLabel* multnsdlabel;
    QLabel* seedlabel;
    QLineEdit* seed;
    QLabel* seedlabel1;
    QLabel* backgroundlabel;
    QLineEdit* backgr1;
    QLabel* backgr1label;
    QLineEdit* backgr2;
    QLabel* backgr2label;
    QLineEdit* backgr3;
    QLabel* backgr3label;
    QLineEdit* backgr4;
    QLabel* backgr4label;
    QLineEdit* backgr5;
    QLabel* backgr5label;
    QLineEdit* backgr6;
    QLabel* backgr6label;
    QLineEdit* backgr7;
    QLabel* backgr7label; 

    QFrame* Line4;
    QPushButton* okbutton;
    QPushButton* cancelbutton;
 private:
    void populate_idheader();
    void populate_geometry();
    void populate_measurement();

    bool check_idheader(QString& errs);
    //    bool check_geometry(QString& errs);
    //    bool check_measurement(QString& errs);

    QString get_idheader();
    QString get_geometry();
    QString get_measurement();

private slots:
    void deactivateFields();
    void activatePseudoFields();
    void activateRealOptions();

    // for set 6
    void activateParallelFields();
    void activateDivergentFields();
    void activateUserFields();
    void activateProgramFields();
    void activateEqualSpacingFields();
    void deactivateEqualSpacingFields();
    void deactivateNoisyFields();
    void activateNoisyFields();
    void activateQuantumFields();
    void activateScatterFields();
    void activateAdditiveFields();
    void activateMultiplicativeFields();

    // for set 7

    // for everything (mainly if pseudo)
    void checkValues();
};

#endif // PROJECTIONWINDOW_H
