/** @file trace.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "trace.h"

#include <qbuttongroup.h>
#include <qlabel.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>

/** 
 *  Constructs a tracewindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
tracewindow::tracewindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
  int width = 340;
    if ( !name )
	setName( "tracewindow" );
    resize( width, 270 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( width, 270 ) );
    setMaximumSize( QSize( width, 270 ) );
    setCaption( tr( "Trace Window" ) );

    tracelabel = new QLabel( this, "tracelabel" );
    tracelabel->setGeometry( QRect( 10, 10, 90, 30 ) ); 
    tracelabel->setMinimumSize( QSize( 90, 30 ) );
    tracelabel->setMaximumSize( QSize( 90, 30 ) );
    tracelabel->setText( tr( "TRACE" ) );

    Line18 = new QFrame( this, "Line18" );
    Line18->setGeometry( QRect( -5, 190, width + 10, 16 ) ); 
    Line18->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    tracebuttongroup = new QButtonGroup( this, "tracebuttongroup" );
    tracebuttongroup->setGeometry( QRect( 80, 10, 200, 180 ) ); 
    tracebuttongroup->setTitle( tr( "trace_level" ) );
    tracebuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    tracebuttonslayout = new QGridLayout( tracebuttongroup, 5, 2 );

    tl0button = new QRadioButton( tracebuttongroup, "tl0button" );
    tl0button->setChecked( TRUE );
    tl0button->setText( tr( "0 (unspecified)" ) );
    // rows start from 0: leave some space on top
    tracebuttonslayout->addMultiCellWidget(tl0button, 1, 1, 0, 1, Qt::AlignHCenter | Qt::AlignTop );

    tl1button = new QRadioButton( tracebuttongroup, "tl1button" );
    tl1button->setText( tr( "1" ) );
    tracebuttonslayout->addWidget(tl1button, 2, 0, Qt::AlignHCenter | Qt::AlignVCenter );

    tl5button = new QRadioButton( tracebuttongroup, "tl5button" );
    tl5button->setText( tr( "5" ) );
    tracebuttonslayout->addWidget(tl5button, 2, 1, Qt::AlignHCenter | Qt::AlignVCenter );

    tl6button = new QRadioButton( tracebuttongroup, "tl6button" );
    tl6button->setText( tr( "6" ) );
    tracebuttonslayout->addWidget(tl6button, 3, 0, Qt::AlignHCenter | Qt::AlignVCenter );

    tl7button = new QRadioButton( tracebuttongroup, "tl7button" );
    tl7button->setText( tr( "7" ) );
    tracebuttonslayout->addWidget(tl7button, 3, 1, Qt::AlignHCenter | Qt::AlignVCenter );

    tl8button = new QRadioButton( tracebuttongroup, "tl8button" );
    tl8button->setText( tr( "8" ) );
    tracebuttonslayout->addWidget(tl8button, 4, 0, Qt::AlignHCenter | Qt::AlignVCenter );

    tl10button = new QRadioButton( tracebuttongroup, "tl10button" );
    tl10button->setText( tr( "10" ) );
    tracebuttonslayout->addWidget(tl10button, 4, 1, Qt::AlignHCenter | Qt::AlignVCenter );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 210, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(accept()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( width - 150, 210, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
tracewindow::~tracewindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Returns a string for TRACE input sequence
@param void
@return trace
@author Bruno M. Carvalho
@version 1.0 */
QString tracewindow::getOutput()
{
  QString selectedText = tracebuttongroup->selected()->text();
  QString traceText = (selectedText.startsWith("0")) ? "0" : selectedText;
  QString output = "TRACE " + traceText + " ";
  return output;
}
